//Copyright Paul Reiche, Fred Ford. 1992-2002

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

// JMS 2009: -Made Alpha Vulpeculae Orz space portal
//			 -Added a system into Orz space starmap_array
// JMS 2010: -Added Lurg home system
//			 -Added shofixti crash site
//			 -Added possible systems for those who mysteriously vanished in a hurry...
//			 -Added little something to Gamma Janus

// XXX: for XXX_DEFINED constants
#include "encount.h"
#include "resinst.h"
#include "planets/planets.h"
#include "planets/elemdata.h"


// NOTE: each block of stars (hyperspace, quasispace, etc.) MUST be
// ordered by the y-coordinate (from smallest to biggest) for the 
// FindStar binary search to work!  Typical symptom of it NOT 
// working is flying up to a star but not being sucked into it.
STAR_DESC starmap_array[] =
{
//                           postfix name index (like 'Normae')
//                         prefix name index (like 'Alpha')   |
//                                        alien presence  |   |
//                                     owner (unused)  |  |   |
//   x,    y              star type   colour       |   |  |   |
{{5207,  83}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 3, 29},
{{4256, 122}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 0, 25},
{{7703, 145}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 1, 115},
{{1827, 172}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 3, 104},
{{3468, 182}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 1, 17},
{{5314, 210}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 2, 29},
{{9044, 228}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 1, 49},
{{1325, 230}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), MELNORME_HOME_DEFINED, 2, 104},  //TODO pick a better star somewhere round here
{{6886, 317}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 2, 36},
{{1877, 355}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 1, 104},
{{3254, 357}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 4, 17},
{{9246, 368}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 2, 56},
{{3456, 418}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 2, 17},
{{6749, 429}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 1, 36},
{{2379, 458}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 4, 104},
{{9862, 467}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 0, 32},
{{5482, 480}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 6, 29},
{{5184, 485}, MAKE_STAR (GIANT_STAR, BLUE_BODY, -1), 0, 1, 29},
{{4817, 501}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 7, 29},
{{5386, 520}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 5, 29},
{{7727, 525}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 2, 115},
{{3256, 528}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 3, 17},
{{ 606, 602}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 1, 58},
{{3717, 656}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 2, 59},
{{ 702, 660}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 2, 58},
{{4123, 667}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 4, 28},
{{8560, 707}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 3, 49},
{{8012, 712}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 3, 115},
{{1162, 724}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 12, 43},
{{5664, 727}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), RAINBOW_DEFINED, 4, 29},
{{1275, 743}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 11, 43},
{{1547, 761}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 10, 43},
{{6635, 766}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 2, 77},
{{6462, 789}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 1, 77},
{{8844, 790}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 2, 49},
{{3524, 796}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 1, 59},
{{3999, 812}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 1, 28},
{{7791, 819}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 4, 115},
{{1570, 846}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 9, 43},
{{3759, 879}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 5, 28},
{{1305, 894}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 15, 43},
{{1402, 902}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 2, 43},
{{4009, 923}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 2, 28},
{{6670, 935}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 3, 77},
{{1190, 943}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 14, 43},
{{6230, 968}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 4, 77},
{{6882, 989}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 9, 53},
{{1233, 997}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 13, 43},
{{1338,1043}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 3, 43},
{{4925,1043}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 3, 123},
{{4161,1075}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 3, 28},
{{5035,1082}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 2, 123},
{{1567,1104}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 8, 43},
{{1279,1132}, MAKE_STAR (SUPER_GIANT_STAR, ORANGE_BODY, -1), MELNORME0_DEFINED, 1, 43},
{{  66,1134}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 0, 70},
{{1355,1137}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 4, 43},
{{5229,1140}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 1, 123},
{{3422,1144}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 1, 12},
{{6915,1174}, MAKE_STAR (SUPER_GIANT_STAR, GREEN_BODY, -1), MELNORME1_DEFINED, 1, 53},
{{3294,1184}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 2, 12},
{{4335,1207}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 1, 71},
{{8610,1218}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 2, 34},
{{4495,1227}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 2, 71},
{{6966,1231}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 4, 53},
{{2816,1241}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 3, 111},
{{1510,1247}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 5, 43},
{{9654,1252}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 2, 68},
{{2927,1304}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 1, 111},
{{1186,1305}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 7, 43},
{{9714,1317}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 4, 68},
{{2667,1321}, MAKE_STAR (SUPER_GIANT_STAR, ORANGE_BODY, -1), MELNORME2_DEFINED, 1, 31},
{{6912,1326}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 5, 53},
{{6496,1363}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 1, 54},
{{1602,1364}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 6, 43},
{{3460,1368}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 3, 12},
{{9396,1384}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 6, 68},
{{7275,1396}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 2, 65},
{{6358,1399}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 3, 54},
{{4630,1410}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 3, 42},
{{9880,1422}, MAKE_STAR (GIANT_STAR, GREEN_BODY, -1), 0, 1, 68},
{{9470,1431}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 5, 68},
{{9705,1442}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 3, 68},
{{4890,1450}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 8, 42},
{{6780,1469}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 3, 53},
{{2942,1489}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 2, 111},
{{4570,1490}, MAKE_STAR (GIANT_STAR, WHITE_BODY, -1), 0, 1, 42},
{{6949,1517}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 2, 53},
{{6470,1522}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 2, 54},
{{5669,1539}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 4, 19},
{{4670,1570}, MAKE_STAR (GIANT_STAR, GREEN_BODY, -1), 0, 2, 42},
{{4800,1580}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 7, 42},
{{2750,1610}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 4, 31},
{{6741,1665}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 6, 53},
{{6450,1684}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 8, 54},
{{4740,1700}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 6, 42},
{{6577,1711}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 7, 54},
{{7460,1725}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 1, 65},
{{4640,1740}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 5, 42},
{{9196,1753}, MAKE_STAR (GIANT_STAR, WHITE_BODY, -1), 0, 1, 13},
{{3151,1765}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 2, 31},
{{8286,1772}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 1, 34},
{{5826,1773}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 2, 19},
{{4510,1780}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 4, 42},
{{2337,1822}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 3, 31},
{{9274,1909}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 2, 13},
{{7665,2035}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 3, 65},
{{5627,2062}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 3, 19},
{{6890,2078}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 9, 19},
{{ 338,2148}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 3, 46},
{{5851,2171}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 1, 19},
{{5407,2185}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 5, 19},
{{5079,2233}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 1, 20},
{{ 897,2242}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 2, 46},
{{6001,2288}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 6, 19},
{{8068,2306}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 2, 45},
{{6291,2352}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 0, 15},
{{1442,2373}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 1, 46},
{{9663,2386}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 0, 5},
{{8412,2410}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 3, 45},
{{6654,2446}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 8, 19},
{{8497,2572}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 1, 45},
{{6259,2587}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 7, 19},
{{5259,2595}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 3, 20},
{{5119,2618}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 2, 20},
{{8240,2700}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 5, 45},
{{8513,2719}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 4, 45},
{{7723,2737}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 0, 1},
{{4661,2741}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 5, 69},
{{7003,2790}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), RAINBOW_DEFINED, 1, 67},
{{3180,2811}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 0, 116},
{{3989,2982}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 3, 41},
{{ 132,3027}, MAKE_STAR (SUPER_GIANT_STAR, BLUE_BODY, -1), MELNORME3_DEFINED, 1, 30},
{{1381,3066}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 5, 27},
{{2115,3099}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 0, 131},
{{7495,3100}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 6, 2},
{{1329,3105}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 4, 27},
{{6915,3125}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 2, 67},
{{4916,3127}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 1, 69},
{{1730,3146}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 7, 27},
{{5200,3162}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 2, 69},
{{3964,3177}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 1, 41},
{{1638,3189}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 6, 27},
{{1143,3215}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 3, 27},
{{5491,3218}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 3, 69},
{{ 162,3233}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 2, 30},
{{7533,3271}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 5, 2},
{{3474,3286}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 1, 57},
{{1425,3291}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 1, 27},
{{4173,3300}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 4, 41},
{{6185,3310}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 3, 66},
{{5725,3333}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 4, 69},
{{9330,3338}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 3, 6},
{{3860,3415}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 2, 41},
{{3231,3436}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 2, 57},
{{1486,3453}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 2, 27},
{{2446,3453}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 3, 37},
{{4976,3543}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 1, 8},
{{3013,3571}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 3, 40},
{{3462,3594}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 3, 57},
{{7585,3611}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 4, 2},
{{2538,3619}, MAKE_STAR (GIANT_STAR, BLUE_BODY, -1), 0, 1, 37},
{{1508,3635}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 3, 35},
{{6909,3641}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 3, 61},
{{6317,3646}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 1, 66},
{{2406,3650}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 4, 37},
{{4447,3709}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 6, 8},
{{7610,3744}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 1, 2},
{{6416,3758}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 2, 66},
{{2521,3787}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 2, 37},
{{7045,3789}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), HINT_DEFINED, 2, 61},  // JMS: Someone was here, but left in a hurry... possibility #1
{{6940,3797}, MAKE_STAR (GIANT_STAR, YELLOW_BODY, -1), 0, 1, 61},
{{7768,3812}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 2, 2},
{{8892,3813}, MAKE_STAR (GIANT_STAR, RED_BODY, -1), 0, 1, 6},
{{4800,3820}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 3, 8},
{{4750,3850}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 4, 8},
{{9074,3852}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 2, 6},
{{6835,3874}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 4, 61},
{{2689,3908}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 5, 37},
{{6615,3930}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 3, 7},
{{2376,3941}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 6, 37},
{{3062,3951}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 1, 40},
{{4942,3957}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 2, 8},
{{6480,3970}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), HINT_DEFINED, 4, 7}, // JMS: Someone was here, but left in a hurry... possibility #2
{{1471,4000}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 2, 35},
{{ 239,4026}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 2, 113},
{{9142,4075}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 4, 6},
{{4690,4080}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 5, 8},
{{2821,4083}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 2, 40},
{{7657,4083}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 3, 2},
{{6667,4100}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 2, 7},
{{6370,4110}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 5, 7},
{{5855,4125}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 0, 0},
{{ 441,4172}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 1, 113},
{{6564,4183}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), HINT_DEFINED, 1, 7}, // JMS: Someone was here, but left in a hurry... possibility #3
{{  87,4190}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 3, 113},
{{1239,4228}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 1, 35},
{{3822,4230}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 1, 112},
{{6510,4276}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 6, 7},
{{6375,4300}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 9, 7},
{{6615,4310}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 7, 7},
{{3002,4326}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 4, 40},
{{9786,4377}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 2, 39},
{{3710,4387}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 2, 112}, // JMS: This baby should have LURG_DEFINED in full game...
{{6540,4400}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 8, 7},
{{9680,4495}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 3, 39},
{{2251,4502}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 0, 121},
{{ 171,4516}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 4, 113},
{{9870,4573}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 1, 39},
{{4445,4626}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 1, 103},
{{6099,4776}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 3, 119},
{{6828,4781}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 3, 50},
{{7130,4790}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 1, 50},
{{ 396,4799}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 5, 113},
{{1738,4812}, MAKE_STAR (GIANT_STAR, WHITE_BODY, -1), 0, 0, 52},
{{8342,4853}, MAKE_STAR (GIANT_STAR, BLUE_BODY, -1), RAINBOW_DEFINED, 0, 101},
{{5807,4872}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 4, 119},
{{4395,4908}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 2, 103},
{{9023,4920}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 1, 44},
{{ 808,4930}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 0, 62},
{{5882,4940}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 1, 119},
{{6816,4970}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 2, 50},
{{6074,4989}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 2, 119},
{{ 674,5055}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 6, 113},
{{1216,5057}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 5, 51},
{{9062,5083}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 2, 44},
{{ 440,5236}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 4, 73},
{{5977,5246}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 5, 119},
{{4177,5248}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 3, 103},
{{1213,5295}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 3, 51},
{{1164,5307}, MAKE_STAR (GIANT_STAR, ORANGE_BODY, -1), 0, 1, 51},
{{  58,5311}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 5, 73},
{{ 569,5314}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 3, 73},
{{1253,5342}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 2, 51},
{{6270,5479}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 3, 21},
{{2351,5516}, MAKE_STAR (SUPER_GIANT_STAR, RED_BODY, -1), MELNORME4_DEFINED, 1, 11},
{{4688,5537}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 3, 64},
{{3759,5572}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 2, 18},
{{1105,5575}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 4, 51},
{{7266,5623}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 1, 33},
{{2881,5630}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 8, 18},
{{6014,5632}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 2, 21},
{{4416,5633}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), SHOFIXTI_CRASH_SITE_DEFINED, 2, 64}, // JMS
{{3537,5652}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 5, 18},
{{8066,5653}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 3, 72},
{{ 822,5659}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 2, 73},
{{9420,5664}, MAKE_STAR (GIANT_STAR, ORANGE_BODY, -1), 0, 1, 47},
{{5479,5686}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 1, 55},
{{7862,5691}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 4, 72},
{{7467,5714}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 1, 72},
{{2382,5718}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 2, 11},
{{9510,5758}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 2, 47},
{{ 385,5774}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 1, 73},
{{4699,5780}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 1, 64},
{{8123,5784}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 2, 72},
{{6107,5785}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 1, 21},
{{9645,5791}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 4, 47},
{{7704,5794}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 5, 72},
{{5316,5817}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 2, 55},
{{3398,5840}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 6, 18},
{{7312,5841}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 3, 33},
{{3855,5853}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 1, 18},
{{9388,5863}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), HINT2_DEFINED, 3, 47}, // JMS: Gamma Janus, Yehat hint about precursor artifact
{{6301,5875}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 5, 21},
{{7398,5889}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 2, 33},
{{3974,5892}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 4, 18},
{{3183,5901}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 7, 18},
{{9864,5911}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 10, 74},
{{5437,5916}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 3, 55},
{{9564,5930}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 5, 47},
{{6200,5935}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 4, 21},
{{2304,5954}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 3, 11},
{{6429,5958}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 6, 21},
{{3842,6030}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 3, 18},
{{5708,6041}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 7, 91},
{{4884,6048}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 4, 64},
{{9714,6078}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 7, 74},
{{7187,6083}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 0, 126},
{{7814,6089}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), SHOFIXTI_COLONY_DEFINED, 2, 82},
{{9244,6091}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 0, 125},
{{4923,6101}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 5, 64},
{{7939,6116}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 3, 82},
{{2100,6122}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 4, 11},
{{7771,6146}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 1, 82},
{{5265,6156}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 5, 92},
{{9529,6169}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 8, 74},
{{9911,6180}, MAKE_STAR (GIANT_STAR, ORANGE_BODY, -1), 0, 1, 74},
{{1920,6193}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 5, 11},
{{9747,6221}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 4, 74},
{{1731,6249}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 6, 11},
{{9861,6262}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 2, 74},
{{7908,6269}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), SHOFIXTI_DEFINED, 4, 82},
{{6855,6270}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 5, 81},
{{5570,6289}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 4, 92},
{{9923,6294}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), YEHAT_DEFINED, 3, 74},
{{4967,6295}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 6, 64},
{{7820,6301}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 5, 82},
{{2761,6303}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 9, 38},
{{1296,6322}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 7, 11},
{{6116,6334}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 6, 91},
{{5803,6337}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 3, 91},
{{6787,6338}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 6, 81},
{{5877,6340}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 4, 91},
{{5843,6380}, MAKE_STAR (GIANT_STAR, ORANGE_BODY, -1), 0, 1, 91},
{{9872,6408}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 5, 74},
{{6740,6423}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 7, 81},
{{9596,6429}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 9, 74},
{{5843,6431}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 2, 91},
{{7156,6440}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 4, 81},
{{7004,6441}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 1, 81},
{{5530,6442}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 2, 92},
{{5958,6468}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 5, 91},
{{7058,6475}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 2, 81},
{{5304,6477}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 1, 92},
{{5522,6525}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), PKUNK_DEFINED, 3, 92},
{{7100,6554}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 3, 81},
{{2785,6560}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 8, 38},
{{5134,6565}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 6, 92},
{{3980,6592}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 1, 63},
{{3085,6674}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 2, 38},
{{7411,6718}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 1, 9},
{{3160,6731}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 3, 38},
{{7589,6741}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 2, 9},
{{5675,6742}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 8, 91},
{{1796,6763}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 3, 26},
{{6463,6779}, MAKE_STAR (GIANT_STAR, RED_BODY, -1), 0, 6, 80},
{{8089,6782}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 4, 9},
{{2917,6787}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 7, 38},
{{7854,6787}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 3, 9},
{{8333,6801}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 5, 9},
{{3075,6818}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 1, 38},
{{3223,6854}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 4, 38},
{{5242,6857}, MAKE_STAR (GIANT_STAR, ORANGE_BODY, -1), 0, 3, 90},
{{6515,6866}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 5, 80},
{{9770,6895}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 5, 75},
{{6412,6905}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 2, 80},
{{9681,6916}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), RAINBOW_KNOWN_DEFINED, 6, 75},
{{5230,6952}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 2, 90},
{{5146,6955}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 1, 90},
{{9873,6968}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 4, 75},
{{3043,6969}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 6, 38},
{{3136,6978}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 5, 38},
{{4078,6978}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 2, 63},
{{6559,6993}, MAKE_STAR (SUPER_GIANT_STAR, RED_BODY, -1), MELNORME5_DEFINED, 1, 80},
{{1508,6994}, MAKE_STAR (GIANT_STAR, ORANGE_BODY, -1), 0, 1, 26},
{{6895,7041}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 2, 93},
{{1830,7060}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 2, 26},
{{9337,7066}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 1, 75},
{{8732,7067}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 0, 122},
{{6579,7115}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 3, 80},
{{9875,7145}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 3, 75},
{{4260,7175}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 3, 63},
{{9604,7187}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 2, 75},
{{6312,7260}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 4, 80},
{{6916,7270}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 1, 93},
{{5416,7301}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 0, 120},
{{8958,7354}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 2, 89},
{{9000,7363}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 1, 89},
{{6752,7450}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), SOL_DEFINED, 0, 129},
{{3025,7480}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 6, 48},
{{7187,7500}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 0, 127},
{{6806,7507}, MAKE_STAR (GIANT_STAR, WHITE_BODY, -1), 0, 0, 128},
{{9333,7562}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 3, 88},
{{ 596,7610}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 3, 4},
{{7881,7614}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 2, 96},
{{3743,7623}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 3, 48},
{{9250,7645}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 1, 88},
{{5650,7646}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 7, 85},
{{1485,7651}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 4, 26},
{{4829,7666}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 4, 10},
{{7840,7676}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 1, 96},
{{9333,7687}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), VUX_DEFINED, 2, 88},
{{4880,7698}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 2, 10},
{{3260,7724}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 5, 48},
{{5736,7737}, MAKE_STAR (GIANT_STAR, BLUE_BODY, -1), 0, 6, 85},
{{4654,7745}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 3, 10},
{{6601,7746}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 1, 94},
{{4841,7764}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 5, 10},
{{3527,7768}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 2, 48},
{{5951,7770}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 1, 85},
{{1772,7783}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 5, 26},
{{ 570,7790}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 4, 4},
{{4947,7805}, MAKE_STAR (GIANT_STAR, GREEN_BODY, -1), 0, 1, 10},
{{3418,7809}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 4, 48},
{{6666,7812}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 2, 94},
{{3755,7823}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 2, 24},
{{5705,7838}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 5, 85},
{{ 832,7841}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 2, 4},
{{6140,7847}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 2, 85},
{{3845,7881}, MAKE_STAR (GIANT_STAR, RED_BODY, -1), 0, 1, 24},
{{7791,7895}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 3, 96},
{{6048,7919}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 3, 85},
{{3487,7924}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 8, 48},
{{8678,7926}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 2, 99},
{{8345,7931}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), START_COLONY_DEFINED, 0, 98},
{{ 180,7940}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 3, 60},
{{1040,7940}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 1, 4},
{{8352,7940}, MAKE_STAR (SUPER_GIANT_STAR, WHITE_BODY, -1), MELNORME6_DEFINED, 0, 97},
{{5977,7953}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 4, 85},
{{9221,7986}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), MAIDENS_DEFINED, 1, 100},
{{ 640,7992}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 5, 4},
{{9500,8000}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 2, 100},
{{3690,8065}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 7, 48},
{{ 340,8066}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 2, 60},
{{7104,8083}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 0, 118},
{{ 199,8140}, MAKE_STAR (SUPER_GIANT_STAR, WHITE_BODY, -1), MELNORME7_DEFINED, 1, 60},
{{2114,8140}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 6, 26},
{{5270,8187}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 1, 79},
{{ 509,8190}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 2, 16},
{{5125,8229}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 2, 79},
{{ 370,8240}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 5, 16},
{{5312,8250}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 3, 79},
{{8884,8262}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 3, 99},
{{5742,8268}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), CHMMR_DEFINED, 0, 117},
{{7306,8285}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 1, 95},
{{7402,8309}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 2, 95},
{{8551,8320}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 1, 99},
{{8354,8354}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 4, 99},
{{3349,8362}, MAKE_STAR (SUPER_GIANT_STAR, YELLOW_BODY, -1), MELNORME8_DEFINED, 1, 48},
{{7298,8385}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 3, 95},
{{ 523,8412}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 1, 16},
{{5217,8509}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 4, 78},
{{8641,8512}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 2, 86},
{{3962,8535}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 4, 102},
{{8713,8537}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), ORZ_DEFINED, 3, 86},
{{8587,8566}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), ANDROSYNTH_DEFINED, 7, 86},			 // JMS: Truespace Androsynth are here
{{8654,8587}, MAKE_STAR (SUPER_GIANT_STAR, BLUE_BODY, -1), ORZ_SPACE_PORTAL_DEFINED, 1, 86}, // JMS: Orz space portal is in here!
{{8721,8619}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), TAALO_PROTECTOR_DEFINED, 4, 86},
{{8608,8637}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 6, 86},
{{8499,8648}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 1, 87},
{{5229,8666}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 1, 78},
{{8668,8666}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 5, 86},
{{ 617,8669}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 3, 16},
{{ 419,8677}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 4, 16},
{{4841,8686}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 9, 78},
{{5138,8696}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 5, 78},
{{4092,8717}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 3, 102},
{{4895,8732}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 8, 78},
{{7458,8750}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 6, 106},
{{5351,8758}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 2, 78},
{{8759,8778}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 4, 87},
{{8400,8804}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 2, 87},
{{8619,8830}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 3, 87},
{{7208,8854}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 7, 106},
{{5672,8863}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 7, 78},
{{5167,8875}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 6, 78},
{{ 481,8882}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 6, 16},
{{4119,8882}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 2, 102},
{{9030,8887}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 2, 105},
{{5384,8900}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 3, 78},
{{7727,8951}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 5, 106},
{{9645,8958}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 3, 105},
{{7011,9043}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 8, 106},
{{2246,9092}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 1, 3},
{{4790,9093}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 0, 22},
{{2190,9125}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 2, 3},
{{4165,9152}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 1, 102},
{{7354,9166}, MAKE_STAR (GIANT_STAR, YELLOW_BODY, -1), 0, 4, 106},
{{8833,9187}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 1, 105},
{{1294,9221}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 2, 23},
{{5291,9250}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 1, 84},
{{5501,9259}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 2, 84},
{{5791,9270}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 3, 84},
{{7354,9291}, MAKE_STAR (SUPER_GIANT_STAR, RED_BODY, -1), MELNORME3_DEFINED, 1, 106},
{{6104,9333}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 4, 84},
{{7687,9333}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 3, 106},
{{2374,9335}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 3, 3},
{{4239,9347}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 5, 102},
{{3913,9353}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 6, 102},
{{8187,9375}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 2, 107},
{{1545,9392}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 1, 23},
{{6758,9418}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 2, 108},
{{7520,9437}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 2, 106},
{{8000,9500}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 1, 107},
{{4083,9504}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 7, 102},
{{5149,9519}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 5, 76},
{{7148,9551}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 2, 109},
{{ 616,9613}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 1, 130},
{{5090,9614}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 6, 76},
{{7770,9625}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 4, 107},
{{5267,9645}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 3, 76},
{{6604,9645}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 1, 108},
{{7274,9663}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 1, 109},
{{5229,9666}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), ILWRATH_DEFINED, 1, 76},
{{8083,9674}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 3, 107},
{{7416,9687}, MAKE_STAR (GIANT_STAR, ORANGE_BODY, -1), SPATHI_DEFINED, 5, 109},
{{ 801,9707}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 2, 130},
{{7250,9708}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 3, 109},
{{5288,9735}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 2, 76},
{{4429,9739}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), 0, 1, 14},
{{7354,9741}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 4, 109},
{{7583,9750}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 6, 109},
{{5166,9770}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 4, 76},
{{9125,9770}, MAKE_STAR (DWARF_STAR, BLUE_BODY, -1), SYREEN_DEFINED, 0, 114},
{{4470,9815}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 2, 14},
{{4862,9823}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 3, 124},
{{4960,9858}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 4, 124},
{{4892,9870}, MAKE_STAR (GIANT_STAR, RED_BODY, -1), 0, 1, 124},
{{7145,9916}, MAKE_STAR (DWARF_STAR, GREEN_BODY, -1), 0, 1, 110},
{{4889,9938}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 2, 124},
{{7479,9958}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 7, 109},
{{5926,9972}, MAKE_STAR (DWARF_STAR, RED_BODY, -1), 0, 2, 83},
{{7062,9991}, MAKE_STAR (DWARF_STAR, ORANGE_BODY, -1), 0, 2, 110},
	

	{{MAX_X_UNIVERSE << 1, MAX_Y_UNIVERSE << 1}, 0, 0, 0, 0},

	// QuasiSpace locations
#define VORTEX_SCALE 20
	{{(-12* VORTEX_SCALE) + 5000, (-21 * VORTEX_SCALE) + 5000},
			MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 0, 132},
	{{( 1 * VORTEX_SCALE) + 5000, (-20 * VORTEX_SCALE) + 5000},
			MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 0, 132},
	{{(-16 * VORTEX_SCALE) + 5000, (-18 * VORTEX_SCALE) + 5000},
			MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 0, 132},
	{{( 8 * VORTEX_SCALE) + 5000, (-17 * VORTEX_SCALE) + 5000},
			MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 0, 132},
	{{( 3 * VORTEX_SCALE) + 5000, (-13 * VORTEX_SCALE) + 5000},
			MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 0, 132},
	{{(-21 * VORTEX_SCALE) + 5000, (-4 * VORTEX_SCALE) + 5000},
			MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 0, 132},
	{{(-4 * VORTEX_SCALE) + 5000, (-4 * VORTEX_SCALE) + 5000},
			MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 0, 132},
	{{(-12 * VORTEX_SCALE) + 5000, (-2 * VORTEX_SCALE) + 5000},
			MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 0, 132},
	{{(-26 * VORTEX_SCALE) + 5000, (2 * VORTEX_SCALE) + 5000},
			MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 0, 132},
	{{(-17 * VORTEX_SCALE) + 5000, (7 * VORTEX_SCALE) + 5000},
			MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 0, 132},
	{{(10 * VORTEX_SCALE) + 5000, (7 * VORTEX_SCALE) + 5000},
			MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 0, 132},
	{{(15 * VORTEX_SCALE) + 5000, (14 * VORTEX_SCALE) + 5000},
			MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 0, 132},
	{{(22 * VORTEX_SCALE) + 5000, (16 * VORTEX_SCALE) + 5000},
			MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 0, 132},
	{{(-6 * VORTEX_SCALE) + 5000, (19 * VORTEX_SCALE) + 5000},
			MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 0, 132},
	{{(10 * VORTEX_SCALE) + 5000, (20 * VORTEX_SCALE) + 5000},
			MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 0, 132},
	{{6134, 5900}, MAKE_STAR (DWARF_STAR, YELLOW_BODY, -1), 0, 0, 132},
	{{MAX_X_UNIVERSE << 1, MAX_Y_UNIVERSE << 1}, 0, 0, 0, 0},
	
	// JMS: ORZ space locations
	{{5000, 5000}, MAKE_STAR (DWARF_STAR, WHITE_BODY, -1), 0, 0, 132},
	{{MAX_X_UNIVERSE << 1, MAX_Y_UNIVERSE << 1}, 0, 0, 0, 0},
};

const BYTE element_array[NUMBER_OF_ELEMENTS] =
{
	COMMON, /* HYDROGEN */
	COMMON, /* HELIUM */
	COMMON, /* LITHIUM */
	BASE_METAL, /* BERYLLIUM */
	BASE_METAL, /* BORON */
	COMMON, /* CARBON */
	COMMON, /* NITROGEN */
	CORROSIVE, /* OXYGEN */
	CORROSIVE, /* FLUORINE */
	NOBLE, /* NEON */
	BASE_METAL, /* SODIUM */
	BASE_METAL, /* MAGNESIUM */
	BASE_METAL, /* ALUMINUM */
	COMMON, /* SILICON */
	COMMON, /* PHOSPHORUS */
	CORROSIVE, /* SULFUR */
	CORROSIVE, /* CHLORINE */
	NOBLE, /* ARGON */
	BASE_METAL, /* POTASSIUM */
	BASE_METAL, /* CALCIUM */
	BASE_METAL, /* SCANDIUM */
	BASE_METAL, /* TITANIUM */
	BASE_METAL, /* VANADIUM */
	BASE_METAL, /* CHROMIUM */
	BASE_METAL, /* MANGANESE */
	BASE_METAL, /* IRON */
	BASE_METAL, /* COBALT */
	BASE_METAL, /* NICKEL */
	BASE_METAL, /* COPPER */
	BASE_METAL, /* ZINC */
	BASE_METAL, /* GALLIUM */
	BASE_METAL, /* GERMANIUM */
	COMMON, /* ARSENIC */
	COMMON, /* SELENIUM */
	CORROSIVE, /* BROMINE */
	NOBLE, /* KRYPTON */
	BASE_METAL, /* RUBIDIUM */
	BASE_METAL, /* STRONTIUM */
	BASE_METAL, /* YTTRIUM */
	BASE_METAL, /* ZIRCONIUM */
	BASE_METAL, /* NIOBIUM */
	BASE_METAL, /* MOLYBDENUM */
	RADIOACTIVE, /* TECHNETIUM */
	BASE_METAL, /* RUTHENIUM */
	BASE_METAL, /* RHODIUM */
	PRECIOUS, /* PALLADIUM */
	PRECIOUS, /* SILVER */
	BASE_METAL, /* CADMIUM */
	BASE_METAL, /* INDIUM */
	BASE_METAL, /* TIN */
	BASE_METAL, /* ANTIMONY */
	BASE_METAL, /* TELLURIUM */
	CORROSIVE, /* IODINE */
	NOBLE, /* XENON */
	BASE_METAL, /* CESIUM */
	BASE_METAL, /* BARIUM */
	RARE_EARTH, /* LANTHANUM */
	RARE_EARTH, /* CERIUM */
	RARE_EARTH, /* PRASEODYMIUM */
	RARE_EARTH, /* NEODYMIUM */
	RARE_EARTH, /* PROMETHIUM */
	RARE_EARTH, /* SAMARIUM */
	RARE_EARTH, /* EUROPIUM */
	RARE_EARTH, /* GADOLINIUM */
	RARE_EARTH, /* TERBIUM */
	RARE_EARTH, /* DYPROSIUM */
	RARE_EARTH, /* HOLMIUM */
	RARE_EARTH, /* ERBIUM */
	RARE_EARTH, /* THULIUM */
	RARE_EARTH, /* YTTERBIUM */
	RARE_EARTH, /* LUTETIUM */
	BASE_METAL, /* HAFNIUM */
	BASE_METAL, /* TANTALUM */
	BASE_METAL, /* TUNGSTEN */
	BASE_METAL, /* RHENIUM */
	BASE_METAL, /* OSMIUM */
	PRECIOUS, /* IRIDIUM */
	PRECIOUS, /* PLATINUM */
	PRECIOUS, /* GOLD */
	BASE_METAL, /* MERCURY */
	BASE_METAL, /* THALLIUM */
	BASE_METAL, /* LEAD */
	BASE_METAL, /* BISMUTH */
	RADIOACTIVE, /* POLONIUM */
	RADIOACTIVE, /* ASTATINE */
	NOBLE, /* RADON */
	RADIOACTIVE, /* FRANCIUM */
	RADIOACTIVE, /* RADIUM */
	RADIOACTIVE, /* ACTINIUM */
	RADIOACTIVE, /* THORIUM */
	RADIOACTIVE, /* PROTACTINIUM */
	RADIOACTIVE, /* URANIUM */
	RADIOACTIVE, /* NEPTUNIUM */
	RADIOACTIVE, /* PLUTONIUM */

	COMMON, /* OZONE */
	COMMON, /* FREE RADICALS */
	COMMON, /* CARBON DIOXIDE */
	COMMON, /* CARBON MONOXIDE */
	COMMON, /* AMMONIA */
	COMMON, /* METHANE */
	COMMON, /* SULFURIC ACID */
	COMMON, /* HYDROCHLORIC ACID */
	COMMON, /* HYDROCYANIC ACID */
	COMMON, /* FORMIC ACID */
	COMMON, /* PHOSPHORIC ACID */
	COMMON, /* FORMALDEHYDE */
	COMMON, /* CYANOACETYLENE */
	COMMON, /* METHANOL */
	COMMON, /* ETHANOL */
	COMMON, /* SILICON MONOXIDE */
	COMMON, /* TITANIUM OXIDE */
	COMMON, /* ZIRCONIUM OXIDE */
	COMMON, /* WATER */
	COMMON, /* SILICON COMPOUNDS */
	COMMON, /* METAL OXIDES */
	EXOTIC, /* QUANTUM BLACK HOLES */
	EXOTIC, /* NEUTRONIUM */
	EXOTIC, /* MAGNETIC MONOPOLES */
	EXOTIC, /* DEGENERATE MATTER */
	EXOTIC, /* SUPER FLUIDS */
	EXOTIC, /* AGUUTI NODULES */
	COMMON, /* IRON COMPOUNDS */
	COMMON, /* ALUMINUM COMPOUNDS */
	COMMON, /* NITROUS OXIDE */
	COMMON, /* RADIOACTIVES */
	COMMON, /* HYDROCARBONS */
	COMMON, /* CARBON COMPOUNDS */
	EXOTIC, /* ANTIMATTER */
	EXOTIC, /* CHARON DUST */
	EXOTIC, /* REISBURG HELICES */
	EXOTIC, /* TZO CRYSTALS */
	COMMON, /* CALCIUM COMPOUNDS */
	COMMON, /* NITRIC ACID */
};

/*------------------------------ Global Data ------------------------------ */

#define NO_DEPOSIT 0

#define TRACE_USEFUL MINERAL_DEPOSIT (FEW, LIGHT)
#define LIGHT_USEFUL MINERAL_DEPOSIT (MODERATE, LIGHT)
#define MEDIUM_USEFUL MINERAL_DEPOSIT (MODERATE, MEDIUM)
#define HEAVY_USEFUL MINERAL_DEPOSIT (MODERATE, HEAVY)
#define HUGE_USEFUL MINERAL_DEPOSIT (NUMEROUS, HEAVY)

const PlanetFrame planet_array[NUMBER_OF_PLANET_TYPES] =
{
	{ /* OOLITE_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + CRATERED_ALGO,
				VIOLET_BODY), /* Color and type/size of planet */
		LOW_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (HIGH_DENSITY, HEAVY), /* Atmosphere and density */
		{
			{HOLMIUM, MEDIUM_USEFUL},
			{ERBIUM, MEDIUM_USEFUL},
			{THULIUM, MEDIUM_USEFUL},
			{YTTERBIUM, MEDIUM_USEFUL},
			{LUTETIUM, MEDIUM_USEFUL},
			{PALLADIUM, MEDIUM_USEFUL},
			{SILVER, MEDIUM_USEFUL},
			{IRIDIUM, MEDIUM_USEFUL},
		},
		OOLITE_COLOR_TAB,
		OOLITE_XLAT_TAB,
		230, 2, 200, 150,
	},
	{ /* YTTRIC_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + CRATERED_ALGO,
				VIOLET_BODY), /* Color and type/size of planet */
		LOW_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (HIGH_DENSITY, MEDIUM), /* Atmosphere and density */
		{
			{YTTERBIUM, HUGE_USEFUL},
			{YTTRIUM, HUGE_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		YTTRIC_COLOR_TAB,
		YTTRIC_XLAT_TAB,
		250, 2, 80, 200,
	},
	{ /* QUASI_DEGENERATE_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + TOPO_ALGO,
				GREEN_BODY), /* Color and type/size of planet */
		MED_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (HIGH_DENSITY, NOTHING), /* Atmosphere and density */
		{
			{DEGENERATE_MATTER, MEDIUM_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		QUASI_DEGENERATE_COLOR_TAB,
		QUASI_DEGENERATE_XLAT_TAB,
		500, 1, 0, 160,
	},
	{ /* LANTHANIDE_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + CRATERED_ALGO,
				YELLOW_BODY), /* Color and type/size of planet */
		HIGH_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (HIGH_DENSITY, LIGHT), /* Atmosphere and density */
		{
			{LANTHANUM, MEDIUM_USEFUL},
			{CERIUM, MEDIUM_USEFUL},
			{PRASEODYMIUM, MEDIUM_USEFUL},
			{NEODYMIUM, MEDIUM_USEFUL},
			{PROMETHIUM, MEDIUM_USEFUL},
			{SAMARIUM, MEDIUM_USEFUL},
			{GADOLINIUM, MEDIUM_USEFUL},
			{TERBIUM, MEDIUM_USEFUL},
		},
		LANTHANIDE_COLOR_TAB,
		LANTHANIDE_XLAT_TAB,
		250, 2, 80, 200,
	},
	{ /* TREASURE_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + TOPO_ALGO,
				YELLOW_BODY), /* Color and type/size of planet */
		NO_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (SUPER_DENSITY, LIGHT), /* Atmosphere and density */
		{
			{PALLADIUM, HEAVY_USEFUL},
			{SILVER, HEAVY_USEFUL},
			{SILVER, HEAVY_USEFUL},
			{IRIDIUM, HEAVY_USEFUL},
			{GOLD, HEAVY_USEFUL},
			{PLATINUM, HEAVY_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		TREASURE_COLOR_TAB,
		TREASURE_XLAT_TAB,
		500, 1, 0, 160,
	},
	{ /* UREA_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + CRATERED_ALGO,
				YELLOW_BODY), /* Color and type/size of planet */
		HIGH_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (LOW_DENSITY, HEAVY), /* Atmosphere and density */
		{
			{AMMONIA, LIGHT_USEFUL},
			{FORMALDEHYDE, MEDIUM_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		UREA_COLOR_TAB,
		UREA_XLAT_TAB,
		230, 2, 200, 150,
	},
	{ /* METAL_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + CRATERED_ALGO,
				ORANGE_BODY), /* Color and type/size of planet */
		LOW_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (HIGH_DENSITY, HEAVY), /* Atmosphere and density */
		{
			{IRON, HUGE_USEFUL},
			{NICKEL, HEAVY_USEFUL},
			{VANADIUM, MEDIUM_USEFUL},
			{SILVER, MEDIUM_USEFUL},
			{URANIUM, HEAVY_USEFUL},
			{SULFUR, MEDIUM_USEFUL},
			{COPPER, HEAVY_USEFUL},
			{NOTHING, NO_DEPOSIT},
		},
		METAL_COLOR_TAB,
		METAL_XLAT_TAB,
		230, 2, 200, 150,
	},
	{ /* RADIOACTIVE_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + CRATERED_ALGO,
				ORANGE_BODY), /* Color and type/size of planet */
		NO_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (SUPER_DENSITY, LIGHT), /* Atmosphere and density */
		{
			{ASTATINE, MEDIUM_USEFUL},
			{FRANCIUM, MEDIUM_USEFUL},
			{RADIUM, MEDIUM_USEFUL},
			{ACTINIUM, MEDIUM_USEFUL},
			{THORIUM, MEDIUM_USEFUL},
			{PROTACTINIUM, MEDIUM_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		RADIOACTIVE_COLOR_TAB,
		RADIOACTIVE_XLAT_TAB,
		250, 2, 80, 200,
	},
	{ /* OPALESCENT_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + CRATERED_ALGO,
				CYAN_BODY), /* Color and type/size of planet */
		LOW_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (LOW_DENSITY, NOTHING), /* Atmosphere and density */
		{
			{SAMARIUM, LIGHT_USEFUL},
			{GADOLINIUM, LIGHT_USEFUL},
			{ARGON, LIGHT_USEFUL},
			{LITHIUM, LIGHT_USEFUL},
			{SILICON, LIGHT_USEFUL},
			{ARSENIC, LIGHT_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		OPALESCENT_COLOR_TAB,
		OPALESCENT_XLAT_TAB,
		400, 1, 100, 190,
	},
	{ /* CYANIC_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + TOPO_ALGO,
				GREEN_BODY), /* Color and type/size of planet */
		NO_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (LIGHT_DENSITY, LIGHT), /* Atmosphere and density */
		{
			{CYANOACETYLENE, HUGE_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		CYANIC_COLOR_TAB,
		CYANIC_XLAT_TAB,
		500, 1, 0, 160,
	},
	{ /* ACID_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + CRATERED_ALGO,
				GREEN_BODY), /* Color and type/size of planet */
		LOW_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (NORMAL_DENSITY, LIGHT), /* Atmosphere and density */
		{
			{SULFURIC_ACID, HEAVY_USEFUL},
			{HYDROCHLORIC_ACID, HEAVY_USEFUL},
			{FORMIC_ACID, HEAVY_USEFUL},
			{HYDROCYANIC_ACID, HEAVY_USEFUL},
			{PHOSPHORIC_ACID, HEAVY_USEFUL},
			{NITRIC_ACID, HEAVY_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		ACID_COLOR_TAB,
		ACID_XLAT_TAB,
		250, 2, 80, 200,
	},
	{ /* ALKALI_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + CRATERED_ALGO,
				GREEN_BODY), /* Color and type/size of planet */
		LOW_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (NORMAL_DENSITY, MEDIUM), /* Atmosphere and density */
		{
			{CALCIUM, MEDIUM_USEFUL},
			{BARIUM, MEDIUM_USEFUL},
			{STRONTIUM, MEDIUM_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		ALKALI_COLOR_TAB,
		ALKALI_XLAT_TAB,
		250, 2, 80, 200,
	},
	{ /* HALIDE_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + CRATERED_ALGO,
				GREEN_BODY), /* Color and type/size of planet */
		NO_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (LOW_DENSITY, LIGHT), /* Atmosphere and density */
		{
			{FLUORINE, MEDIUM_USEFUL},
			{BROMINE, MEDIUM_USEFUL},
			{BROMINE, MEDIUM_USEFUL},
			{ASTATINE, MEDIUM_USEFUL},
			{IODINE, MEDIUM_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		HALIDE_COLOR_TAB,
		HALIDE_XLAT_TAB,
		250, 2, 80, 200,
	},
	{ /* GREEN_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + CRATERED_ALGO,
				GREEN_BODY), /* Color and type/size of planet */
		MED_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (NORMAL_DENSITY, HEAVY), /* Atmosphere and density */
		{
			{PRASEODYMIUM, HEAVY_USEFUL},
			{NEODYMIUM, HEAVY_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		GREEN_COLOR_TAB,
		GREEN_XLAT_TAB,
		230, 2, 200, 150,
	},
	{ /* COPPER_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + TOPO_ALGO,
				GREEN_BODY), /* Color and type/size of planet */
		NO_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (HIGH_DENSITY, MEDIUM), /* Atmosphere and density */
		{
			{COPPER, HUGE_USEFUL},
			{COPPER, HUGE_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		COPPER_COLOR_TAB,
		COPPER_XLAT_TAB,
		500, 1, 0, 160,
	},
	{ /* CARBIDE_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + CRATERED_ALGO,
				RED_BODY), /* Color and type/size of planet */
		LOW_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (NORMAL_DENSITY, LIGHT), /* Atmosphere and density */
		{
			{CARBON, HEAVY_USEFUL},
			{CARBON, HEAVY_USEFUL},
			{CARBON, HEAVY_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		CARBIDE_COLOR_TAB,
		CARBIDE_XLAT_TAB,
		400, 1, 100, 190,
	},
	{ /* ULTRAMARINE_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + CRATERED_ALGO,
				BLUE_BODY), /* Color and type/size of planet */
		NO_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (LIGHT_DENSITY, HEAVY), /* Atmosphere and density */
		{
			{KRYPTON, MEDIUM_USEFUL},
			{COBALT, MEDIUM_USEFUL},
			{HOLMIUM, MEDIUM_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		ULTRAMARINE_COLOR_TAB,
		ULTRAMARINE_XLAT_TAB,
		200, 2, 100, 100,
	},
	{ /* NOBLE_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + CRATERED_ALGO,
				BLUE_BODY), /* Color and type/size of planet */
		NO_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (LIGHT_DENSITY, LIGHT), /* Atmosphere and density */
		{
			{NEON, LIGHT_USEFUL},
			{RADON, LIGHT_USEFUL},
			{ARGON, LIGHT_USEFUL},
			{KRYPTON, LIGHT_USEFUL},
			{XENON, LIGHT_USEFUL},
			{HELIUM, LIGHT_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		NOBLE_COLOR_TAB,
		NOBLE_XLAT_TAB,
		250, 2, 80, 200,
	},
	{ /* AZURE_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + CRATERED_ALGO,
				BLUE_BODY), /* Color and type/size of planet */
		MED_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (LOW_DENSITY, MEDIUM), /* Atmosphere and density */
		{
			{INDIUM, LIGHT_USEFUL},
			{MOLYBDENUM, LIGHT_USEFUL},
			{VANADIUM, LIGHT_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		AZURE_COLOR_TAB,
		AZURE_XLAT_TAB,
		230, 2, 200, 150,
	},
	{ /* CHONDRITE_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + CRATERED_ALGO,
				PURPLE_BODY), /* Color and type/size of planet */
		HIGH_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (NORMAL_DENSITY, NOTHING), /* Atmosphere and density */
		{
			{ETHANOL, HEAVY_USEFUL},
			{FREE_RADICALS, HEAVY_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		CHONDRITE_COLOR_TAB,
		CHONDRITE_XLAT_TAB,
		500, 1, 100, 190,
	},
	{ /* PURPLE_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + CRATERED_ALGO,
				PURPLE_BODY), /* Color and type/size of planet */
		LOW_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (LOW_DENSITY, LIGHT), /* Atmosphere and density */
		{
			{RHENIUM, MEDIUM_USEFUL},
			{CADMIUM, MEDIUM_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		PURPLE_COLOR_TAB,
		PURPLE_XLAT_TAB,
		230, 2, 200, 150,
	},
	{ /* SUPER_DENSE_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + TOPO_ALGO,
				PURPLE_BODY), /* Color and type/size of planet */
		HIGH_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (SUPER_DENSITY, HEAVY), /* Atmosphere and density */
		{
			{LEAD, MEDIUM_USEFUL},
			{OSMIUM, MEDIUM_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		SUPER_DENSE_COLOR_TAB,
		SUPER_DENSE_XLAT_TAB,
		500, 1, 0, 160,
	},
	{ /* PELLUCID_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + CRATERED_ALGO,
				PURPLE_BODY), /* Color and type/size of planet */
		NO_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (LOW_DENSITY, LIGHT), /* Atmosphere and density */
		{
			{TZO_CRYSTALS, TRACE_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		PELLUCID_COLOR_TAB,
		PELLUCID_XLAT_TAB,
		400, 1, 100, 190,
	},
	{ /* DUST_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + CRATERED_ALGO,
				RED_BODY), /* Color and type/size of planet */
		LOW_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (LOW_DENSITY, MEDIUM), /* Atmosphere and density */
		{
			{BISMUTH, LIGHT_USEFUL},
			{ALUMINUM, LIGHT_USEFUL},
			{POTASSIUM, LIGHT_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		DUST_COLOR_TAB,
		DUST_XLAT_TAB,
		250, 2, 80, 200,
	},
	{ /* CRIMSON_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + CRATERED_ALGO,
				RED_BODY), /* Color and type/size of planet */
		NO_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (NORMAL_DENSITY, HEAVY), /* Atmosphere and density */
		{
			{BARIUM, LIGHT_USEFUL},
			{BORON, LIGHT_USEFUL},
			{BERYLLIUM, LIGHT_USEFUL},
			{BISMUTH, LIGHT_USEFUL},
			{BROMINE, LIGHT_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		CRIMSON_COLOR_TAB,
		CRIMSON_XLAT_TAB,
		230, 2, 200, 150,
	},
	{ /* CIMMERIAN_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + TOPO_ALGO,
				RED_BODY), /* Color and type/size of planet */
		LOW_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (HIGH_DENSITY, NOTHING), /* Atmosphere and density */
		{
			{METHANE, MEDIUM_USEFUL},
			{AMMONIA, MEDIUM_USEFUL},
			{METHANOL, MEDIUM_USEFUL},
			{LITHIUM, MEDIUM_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		CIMMERIAN_COLOR_TAB,
		CIMMERIAN_XLAT_TAB,
		500, 1, 0, 160,
	},
	{ /* INFRARED_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + CRATERED_ALGO,
				RED_BODY), /* Color and type/size of planet */
		LOW_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (NORMAL_DENSITY, HEAVY), /* Atmosphere and density */
		{
			{MERCURY, HEAVY_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		INFRARED_COLOR_TAB,
		INFRARED_XLAT_TAB,
		400, 1, 100, 190,
	},
	{ /* SELENIC_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + CRATERED_ALGO,
				WHITE_BODY), /* Color and type/size of planet */
		NO_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (LOW_DENSITY, LIGHT), /* Atmosphere and density */
		{
			{IRON, LIGHT_USEFUL},
			{ALUMINUM, LIGHT_USEFUL},
			{CALCIUM, MEDIUM_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		SELENIC_COLOR_TAB,
		SELENIC_XLAT_TAB,
		230, 2, 200, 150,
	},
	{ /* AURIC_WORLD */
		MAKE_BYTE (SMALL_ROCKY_WORLD + TOPO_ALGO,
				YELLOW_BODY), /* Color and type/size of planet */
		NO_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (HIGH_DENSITY, MEDIUM), /* Atmosphere and density */
		{
			{GOLD, HUGE_USEFUL},
			{GOLD, HUGE_USEFUL},
			{GOLD, HUGE_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		AURIC_COLOR_TAB,
		AURIC_XLAT_TAB,
		500, 1, 0, 160,
	},


	{ /* FLUORESCENT_WORLD */
		MAKE_BYTE (LARGE_ROCKY_WORLD + CRATERED_ALGO,
				VIOLET_BODY), /* Color and type/size of planet */
		LOW_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (HIGH_DENSITY, LIGHT), /* Atmosphere and density */
		{
			{TECHNETIUM, HUGE_USEFUL},
			{NEON, HUGE_USEFUL},
			{RADON, LIGHT_USEFUL},
			{POTASSIUM, LIGHT_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		FLUORESCENT_COLOR_TAB,
		FLUORESCENT_XLAT_TAB,
		400, 1, 100, 190,
	},
	{ /* ULTRAVIOLET_WORLD */
		MAKE_BYTE (LARGE_ROCKY_WORLD + CRATERED_ALGO,
				VIOLET_BODY), /* Color and type/size of planet */
		MED_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (HIGH_DENSITY, HEAVY), /* Atmosphere and density */
		{
			{POLONIUM, HUGE_USEFUL},
			{GOLD, HUGE_USEFUL},
			{PHOSPHORUS, HUGE_USEFUL},
			{SCANDIUM, MEDIUM_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		ULTRAVIOLET_COLOR_TAB,
		ULTRAVIOLET_XLAT_TAB,
		250, 2, 80, 200,
	},
	{ /* PLUTONIC_WORLD */
		MAKE_BYTE (LARGE_ROCKY_WORLD + CRATERED_ALGO,
				YELLOW_BODY), /* Color and type/size of planet */
		LOW_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (HIGH_DENSITY, HEAVY), /* Atmosphere and density */
		{
			{BERYLLIUM, HUGE_USEFUL},
			{BORON, HUGE_USEFUL},
			{LANTHANUM, MEDIUM_USEFUL},
			{ASTATINE, MEDIUM_USEFUL},
			{FRANCIUM, MEDIUM_USEFUL},
			{TITANIUM, LIGHT_USEFUL},
			{CERIUM, MEDIUM_USEFUL},
			{NOTHING, NO_DEPOSIT},
		},
		PLUTONIC_COLOR_TAB,
		PLUTONIC_XLAT_TAB,
		250, 2, 80, 200,
	},
	{ /* RAINBOW_WORLD */
		MAKE_BYTE (LARGE_ROCKY_WORLD + TOPO_ALGO,
				YELLOW_BODY), /* Color and type/size of planet */
		NO_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (LOW_DENSITY, LIGHT), /* Atmosphere and density */
		{
			{ACTINIUM, HEAVY_USEFUL},
			{THORIUM, HEAVY_USEFUL},
			{PROTACTINIUM, HEAVY_USEFUL},
			{NEPTUNIUM, HEAVY_USEFUL},
			{PLUTONIUM, HEAVY_USEFUL},
			{OZONE, HUGE_USEFUL},
			{OZONE, HUGE_USEFUL},
			{NOTHING, NO_DEPOSIT},
		},
		RAINBOW_COLOR_TAB,
		RAINBOW_XLAT_TAB,
		500, 1, 20, 100,
	},
	{ /* SHATTERED_WORLD */
		MAKE_BYTE (LARGE_ROCKY_WORLD + CRATERED_ALGO,
				ORANGE_BODY), /* Color and type/size of planet */
		SUPER_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (NORMAL_DENSITY, NOTHING), /* Atmosphere and density */
		{
			{PALLADIUM, HUGE_USEFUL},
			{IRIDIUM, HUGE_USEFUL},
			{TECHNETIUM, HUGE_USEFUL},
			{POLONIUM, HUGE_USEFUL},
			{SODIUM, HUGE_USEFUL},
			{MANGANESE, HUGE_USEFUL},
			{CHROMIUM, HUGE_USEFUL},
			{NOTHING, NO_DEPOSIT},
		},
		SHATTERED_COLOR_TAB,
		SHATTERED_XLAT_TAB,
		500, 1, 0, 185,
	},
	{ /* SAPPHIRE_WORLD */
		MAKE_BYTE (LARGE_ROCKY_WORLD + TOPO_ALGO,
				CYAN_BODY), /* Color and type/size of planet */
		NO_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (LOW_DENSITY, NOTHING), /* Atmosphere and density */
		{
			{REISBURG_HELICES, HUGE_USEFUL},
			{RT_SUPER_FLUID, MEDIUM_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		SAPPHIRE_COLOR_TAB,
		SAPPHIRE_XLAT_TAB,
		80, 1, 0, 128,
	},
	{ /* ORGANIC_WORLD */
		MAKE_BYTE (LARGE_ROCKY_WORLD + TOPO_ALGO,
				CYAN_BODY), /* Color and type/size of planet */
		MED_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (NORMAL_DENSITY, LIGHT), /* Atmosphere and density */
		{
			{FREE_RADICALS, HEAVY_USEFUL},
			{FORMALDEHYDE, HEAVY_USEFUL},
			{CARBON, HEAVY_USEFUL},
			{CARBON_DIOXIDE, HEAVY_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		ORGANIC_COLOR_TAB,
		ORGANIC_XLAT_TAB,
		500, 1, 0, 160,
	},
	{ /* XENOLITHIC_WORLD */
		MAKE_BYTE (LARGE_ROCKY_WORLD + CRATERED_ALGO,
				CYAN_BODY), /* Color and type/size of planet */
		LOW_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (NORMAL_DENSITY, MEDIUM), /* Atmosphere and density */
		{
			{ALUMINUM, HUGE_USEFUL},
			{PLATINUM, LIGHT_USEFUL},
			{GERMANIUM, TRACE_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		XENOLITHIC_COLOR_TAB,
		XENOLITHIC_XLAT_TAB,
		400, 1, 100, 190,
	},
	{ /* REDUX_WORLD */
		MAKE_BYTE (LARGE_ROCKY_WORLD + CRATERED_ALGO,
				GREEN_BODY), /* Color and type/size of planet */
		LOW_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (NORMAL_DENSITY, LIGHT), /* Atmosphere and density */
		{
			{BROMINE, MEDIUM_USEFUL},
			{OXYGEN, MEDIUM_USEFUL},
			{FLUORINE, MEDIUM_USEFUL},
			{SULFUR, MEDIUM_USEFUL},
			{CHLORINE, MEDIUM_USEFUL},
			{IODINE, MEDIUM_USEFUL},
			{ZIRCONIUM, LIGHT_USEFUL},
			{NOTHING, NO_DEPOSIT},
		},
		REDUX_COLOR_TAB,
		REDUX_XLAT_TAB,
		500, 1, 0, 190,
	},
	{ /* PRIMORDIAL_WORLD */
		MAKE_BYTE (LARGE_ROCKY_WORLD + CRATERED_ALGO,
				GREEN_BODY), /* Color and type/size of planet */
		SUPER_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (NORMAL_DENSITY, HEAVY), /* Atmosphere and density */
		{
			{CESIUM, MEDIUM_USEFUL},
			{BARIUM, MEDIUM_USEFUL},
			{RUBIDIUM, MEDIUM_USEFUL},
			{METHANE, HUGE_USEFUL},
			{AMMONIA, HUGE_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		PRIMORDIAL_COLOR_TAB,
		PRIMORDIAL_XLAT_TAB,
		250, 2, 10, 200,
	},
	{ /* EMERALD_WORLD */
		MAKE_BYTE (LARGE_ROCKY_WORLD + TOPO_ALGO,
				GREEN_BODY), /* Color and type/size of planet */
		NO_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (LOW_DENSITY, NOTHING), /* Atmosphere and density */
		{
			{AGUUTI_NODULES, HUGE_USEFUL},
			{ANTIMATTER, MEDIUM_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		EMERALD_COLOR_TAB,
		EMERALD_XLAT_TAB,
		80, 1, 0, 128,
	},
	{ /* CHLORINE_WORLD */
		MAKE_BYTE (LARGE_ROCKY_WORLD + CRATERED_ALGO,
				GREEN_BODY), /* Color and type/size of planet */
		HIGH_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (NORMAL_DENSITY, LIGHT), /* Atmosphere and density */
		{
			{CHLORINE, HEAVY_USEFUL},
			{CHLORINE, HEAVY_USEFUL},
			{HYDROCHLORIC_ACID, HEAVY_USEFUL},
			{HYDROCHLORIC_ACID, HEAVY_USEFUL},
			{ZINC, LIGHT_USEFUL},
			{GALLIUM, TRACE_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		CHLORINE_COLOR_TAB,
		CHLORINE_XLAT_TAB,
		500, 1, 0, 190,
	},
	{ /* MAGNETIC_WORLD */
		MAKE_BYTE (LARGE_ROCKY_WORLD + CRATERED_ALGO,
				GREEN_BODY), /* Color and type/size of planet */
		HIGH_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (HIGH_DENSITY, NOTHING), /* Atmosphere and density */
		{
			{ZINC, HEAVY_USEFUL},
			{NICKEL, MEDIUM_USEFUL},
			{MAGNETIC_MONOPOLES, TRACE_USEFUL},
			{NIOBIUM, LIGHT_USEFUL},
			{IRON, HEAVY_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		MAGNETIC_COLOR_TAB,
		MAGNETIC_XLAT_TAB,
		400, 1, 100, 190,
	},
	{ /* WATER_WORLD */
		MAKE_BYTE (LARGE_ROCKY_WORLD + CRATERED_ALGO,
				BLUE_BODY), /* Color and type/size of planet */
		LOW_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (NORMAL_DENSITY, MEDIUM), /* Atmosphere and density */
		{
			{IRON, LIGHT_USEFUL},
			{ALUMINUM, LIGHT_USEFUL},
			{TIN, LIGHT_USEFUL},
			{LEAD, LIGHT_USEFUL},
			{URANIUM, TRACE_USEFUL},
			{MOLYBDENUM, TRACE_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		WATER_COLOR_TAB,
		WATER_XLAT_TAB,
		500, 1, 0, 190,
	},
	{ /* TELLURIC_WORLD */
		MAKE_BYTE (LARGE_ROCKY_WORLD + CRATERED_ALGO,
				BLUE_BODY), /* Color and type/size of planet */
		LOW_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (NORMAL_DENSITY, HEAVY), /* Atmosphere and density */
		{
			{IRIDIUM, MEDIUM_USEFUL},
			{RUTHENIUM, MEDIUM_USEFUL},
			{THALLIUM, MEDIUM_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		TELLURIC_COLOR_TAB,
		TELLURIC_XLAT_TAB,
		250, 2, 80, 200,
	},
	{ /* HYDROCARBON_WORLD */
		MAKE_BYTE (LARGE_ROCKY_WORLD + TOPO_ALGO,
				BLUE_BODY), /* Color and type/size of planet */
		LOW_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (NORMAL_DENSITY, LIGHT), /* Atmosphere and density */
		{
			{HYDROCARBONS, HUGE_USEFUL},
			{BISMUTH, LIGHT_USEFUL},
			{TANTALUM, TRACE_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		HYDROCARBON_COLOR_TAB,
		HYDROCARBON_XLAT_TAB,
		500, 1, 0, 160,
	},
	{ /* IODINE_WORLD */
		MAKE_BYTE (LARGE_ROCKY_WORLD + CRATERED_ALGO,
				GREEN_BODY), /* Color and type/size of planet */
		MED_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (LOW_DENSITY, MEDIUM), /* Atmosphere and density */
		{
			{IODINE, HEAVY_USEFUL},
			{MAGNESIUM, LIGHT_USEFUL},
			{TUNGSTEN, TRACE_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		IODINE_COLOR_TAB,
		IODINE_XLAT_TAB,
		230, 2, 200, 150,
	},
	{ /* VINYLOGOUS_WORLD */
		MAKE_BYTE (LARGE_ROCKY_WORLD + CRATERED_ALGO,
				PURPLE_BODY), /* Color and type/size of planet */
		LOW_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (NORMAL_DENSITY, NOTHING), /* Atmosphere and density */
		{
			{TITANIUM, LIGHT_USEFUL},
			{ARSENIC, LIGHT_USEFUL},
			{POTASSIUM, LIGHT_USEFUL},
			{RHENIUM, LIGHT_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		VINYLOGOUS_COLOR_TAB,
		VINYLOGOUS_XLAT_TAB,
		400, 1, 100, 190,
	},
	{ /* RUBY_WORLD */
		MAKE_BYTE (LARGE_ROCKY_WORLD + TOPO_ALGO,
				RED_BODY), /* Color and type/size of planet */
		NO_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (LOW_DENSITY, LIGHT), /* Atmosphere and density */
		{
			{TZO_CRYSTALS, HUGE_USEFUL},
			{NEUTRONIUM, MEDIUM_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		RUBY_COLOR_TAB,
		RUBY_XLAT_TAB,
		80, 1, 0, 128,
	},
	{ /* MAGMA_WORLD */
		MAKE_BYTE (LARGE_ROCKY_WORLD + TOPO_ALGO,
				RED_BODY), /* Color and type/size of planet */
		SUPER_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (HIGH_DENSITY, LIGHT), /* Atmosphere and density */
		{
			{LEAD, LIGHT_USEFUL},
			{NICKEL, LIGHT_USEFUL},
			{IRON, LIGHT_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		MAGMA_COLOR_TAB,
		MAGMA_XLAT_TAB,
		500, 1, 0, 160,
	},
	{ /* MAROON_WORLD */
		MAKE_BYTE (LARGE_ROCKY_WORLD + CRATERED_ALGO,
				RED_BODY), /* Color and type/size of planet */
		NO_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (LOW_DENSITY, MEDIUM), /* Atmosphere and density */
		{
			{CESIUM, MEDIUM_USEFUL},
			{SILICON, MEDIUM_USEFUL},
			{PHOSPHORUS, MEDIUM_USEFUL},
			{RHODIUM, MEDIUM_USEFUL},
			{CADMIUM, MEDIUM_USEFUL},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		MAROON_COLOR_TAB,
		MAROON_XLAT_TAB,
		230, 2, 200, 150,
	},

	{
		MAKE_BYTE (GAS_GIANT + GAS_GIANT_ALGO,
				BLUE_BODY), /* Color and type/size of planet */
		NO_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (GAS_DENSITY, HEAVY), /* Atmosphere and density */
		{
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		BLU_GAS_COLOR_TAB,
		BLU_GAS_XLAT_TAB,
		10, 2, 8, 29,
	},
	{
		MAKE_BYTE (GAS_GIANT + GAS_GIANT_ALGO,
				CYAN_BODY), /* Color and type/size of planet */
		NO_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (GAS_DENSITY, HEAVY), /* Atmosphere and density */
		{
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		CYA_GAS_COLOR_TAB,
		CYA_GAS_XLAT_TAB,
		10, 2, 8, 29,
	},
	{
		MAKE_BYTE (GAS_GIANT + GAS_GIANT_ALGO,
				GREEN_BODY), /* Color and type/size of planet */
		NO_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (GAS_DENSITY, HEAVY), /* Atmosphere and density */
		{
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		GRN_GAS_COLOR_TAB,
		GRN_GAS_XLAT_TAB,
		10, 2, 8, 29,
	},
	{
		MAKE_BYTE (GAS_GIANT + GAS_GIANT_ALGO,
				GRAY_BODY), /* Color and type/size of planet */
		NO_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (GAS_DENSITY, HEAVY), /* Atmosphere and density */
		{
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		GRY_GAS_COLOR_TAB,
		GRY_GAS_XLAT_TAB,
		10, 2, 8, 29,
	},
	{
		MAKE_BYTE (GAS_GIANT + GAS_GIANT_ALGO,
				ORANGE_BODY), /* Color and type/size of planet */
		NO_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (GAS_DENSITY, HEAVY), /* Atmosphere and density */
		{
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		ORA_GAS_COLOR_TAB,
		ORA_GAS_XLAT_TAB,
		10, 2, 8, 29,
	},
	{
		MAKE_BYTE (GAS_GIANT + GAS_GIANT_ALGO,
				PURPLE_BODY), /* Color and type/size of planet */
		NO_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (GAS_DENSITY, HEAVY), /* Atmosphere and density */
		{
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		PUR_GAS_COLOR_TAB,
		PUR_GAS_XLAT_TAB,
		10, 2, 8, 29,
	},
	{
		MAKE_BYTE (GAS_GIANT + GAS_GIANT_ALGO,
				RED_BODY), /* Color and type/size of planet */
		NO_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (GAS_DENSITY, HEAVY), /* Atmosphere and density */
		{
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		RED_GAS_COLOR_TAB,
		RED_GAS_XLAT_TAB,
		10, 2, 8, 29,
	},
	{
		MAKE_BYTE (GAS_GIANT + GAS_GIANT_ALGO,
				VIOLET_BODY), /* Color and type/size of planet */
		NO_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (GAS_DENSITY, HEAVY), /* Atmosphere and density */
		{
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		VIO_GAS_COLOR_TAB,
		VIO_GAS_XLAT_TAB,
		10, 2, 8, 29,
	},
	{ /* A Jupiter-like World */
		MAKE_BYTE (GAS_GIANT + GAS_GIANT_ALGO,
				YELLOW_BODY), /* Color and type/size of planet */
		NO_TECTONICS, /* Tectonics - Scaled with Earth at 82    */
		MAKE_BYTE (GAS_DENSITY, HEAVY), /* Atmosphere and density */
		{
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
			{NOTHING, NO_DEPOSIT},
		},
		YEL_GAS_COLOR_TAB,
		YEL_GAS_XLAT_TAB,
		10, 2, 8, 29,
	},
};

