//Copyright Paul Reiche, Fred Ford. 1992-2002

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

// JMS 2009: Shows *Below* in orz space in the SIS info screen

// JMS_GFX 2011: Merged the resolution Factor stuff from UQM-HD.

#include "sis.h"

#include "colors.h"
#include "races.h"
// XXX: including encount.h for stuff that does not belong there
#include "encount.h"
#include "units.h"
#include "menustat.h"
		// for DrawMenuStateStrings()
#include "gamestr.h"
#include "options.h"
#include "battle.h"
		// For BATTLE_FRAME_RATE
#include "element.h"
#include "setup.h"
#include "state.h"
#include "libs/graphics/gfx_common.h"
#include "libs/tasklib.h"
#include "libs/log.h"

#include <stdio.h>

static StatMsgMode curMsgMode = SMM_DEFAULT;

/**** 1/1-Scale blueprint
static const COUNT crew_lines_sizes[16] =
{
  4,4,3,3,2,1,1,1,2,2,8,7,7,1,2,2
};

static const COORD crew_lines_x_coords[16] = 
{
  93,92,91,90,90,89,90,89,86,85,72,73,73,68,67,67
};

static const COORD crew_lines_y_coords[16] = 
{
  29,27,25,23,21,19,31,29,27,25,23,21,19,23,21,19
};
****/

/**** Enlarged BW blueprint
static const COUNT crew_lines_sizes[16] =
{
  4,3,3,2,2,2,1,1,1,2,9,8,7,2,2,1
};

static const COORD crew_lines_x_coords[16] = 
{
  90,89,88,87,86,85,84,84,83,81,66,67,68,60,61,63
};

static const COORD crew_lines_y_coords[16] = 
{
  20,18,16,14,12,10,8,18,16,14,12,10,8,12,10,8
};
 ****/

/**** Damon's blueprint ****/
static const COUNT crew_lines_sizes[11] =
{
	4,4,3,2,10,9,7,3,3,3,2
};

static const COORD crew_lines_x_coords_1x[16] = 
{
	86,83,82,81,59,60,63,46,49,52,55
};

static const COORD crew_lines_x_coords_4x[16] = 
{
	98,95,94,93,69,70,73,53,56,59,62
};

static const COORD crew_lines_y_coords[16] = 
{
	13,11,9,7,11,9,7,13,11,9,7
};

static const UNICODE *describeWeapon (BYTE moduleType);

void
RepairSISBorder (void)
{
	RECT r;
	CONTEXT OldContext;

	OldContext = SetContext (ScreenContext);

	BatchGraphics ();

	// Left border
	r.corner.x = SIS_ORG_X - 1;
	r.corner.y = SIS_ORG_Y - 1;
	r.extent.width = 1;
	r.extent.height = SIS_SCREEN_HEIGHT + 2;
	SetContextForeGroundColor (SIS_LEFT_BORDER_COLOR);
	DrawFilledRectangle (&r);

	// Right border
	SetContextForeGroundColor (SIS_BOTTOM_RIGHT_BORDER_COLOR);
	r.corner.x += (SIS_SCREEN_WIDTH + 2) - 1;
	DrawFilledRectangle (&r);

	// Bottom border
	r.corner.x = SIS_ORG_X - 1;
	r.corner.y += (SIS_SCREEN_HEIGHT + 2) - 1;
	r.extent.width = SIS_SCREEN_WIDTH + 2;
	r.extent.height = 1;
	DrawFilledRectangle (&r);

	UnbatchGraphics ();

	SetContext (OldContext);
}

void
ClearSISRect (BYTE ClearFlags)
{
	RECT r;
	Color OldColor;
	CONTEXT OldContext;

	OldContext = SetContext (StatusContext);
	OldColor = SetContextForeGroundColor (
			BUILD_COLOR (MAKE_RGB15 (0x0A, 0x0A, 0x0A), 0x08));

	r.corner.x = 2;
	r.extent.width = STATUS_WIDTH - 4;

	BatchGraphics ();
	if (ClearFlags & DRAW_SIS_DISPLAY)
	{
		DeltaSISGauges (UNDEFINED_DELTA, UNDEFINED_DELTA, UNDEFINED_DELTA);
	}

	if (ClearFlags & CLEAR_SIS_RADAR)
	{
		UnlockMutex (GraphicsLock);
		DrawMenuStateStrings ((BYTE)~0, 1);
		LockMutex (GraphicsLock);
#ifdef NEVER
		r.corner.x = RADAR_X - 1;
		r.corner.y = RADAR_Y - 1;
		r.extent.width = RADAR_WIDTH + 2;
		r.extent.height = RADAR_HEIGHT + 2;

		DrawStarConBox (&r, 1,
				BUILD_COLOR (MAKE_RGB15 (0x10, 0x10, 0x10), 0x19),
				BUILD_COLOR (MAKE_RGB15 (0x08, 0x08, 0x08), 0x1F),
				TRUE, BUILD_COLOR (MAKE_RGB15 (0x00, 0x0E, 0x00), 0x6C));
#endif /* NEVER */
	}
	UnbatchGraphics ();

	SetContextForeGroundColor (OldColor);
	SetContext (OldContext);
}

// Draw the SIS title. This is the field at the top of the screen, on the
// right hand side, containing the coordinates in HyperSpace, or the planet
// name in IP.
void
DrawSISTitle (UNICODE *pStr)
{
	TEXT t;
	CONTEXT OldContext;
	RECT r;

	t.baseline.x = SIS_TITLE_WIDTH >> 1;
	t.baseline.y = SIS_TITLE_HEIGHT - RES_CASE(2,6,8); // JMS_GFX
	t.align = ALIGN_CENTER;
	t.pStr = pStr;
	t.CharCount = (COUNT)~0;

	OldContext = SetContext (OffScreenContext);
	r.corner.x = SIS_ORG_X + SIS_SCREEN_WIDTH - SIS_TITLE_BOX_WIDTH + (1 << RESOLUTION_FACTOR); // JMS_GFX
	r.corner.y = SIS_ORG_Y - SIS_TITLE_HEIGHT;
	r.extent.width = SIS_TITLE_WIDTH;
	r.extent.height = SIS_TITLE_HEIGHT - RES_STAT_SCALE(1); // JMS_GFX
	SetContextFGFrame (Screen);
	SetContextClipRect (&r);
	SetContextFont (TinyFont);

	BatchGraphics ();

	// Background color
	SetContextBackGroundColor (SIS_TITLE_BACKGROUND_COLOR);
	ClearDrawable ();

	// Text color
	SetContextForeGroundColor (SIS_TITLE_TEXT_COLOR);
	font_DrawText (&t);

	UnbatchGraphics ();

	SetContextClipRect (NULL);

	SetContext (OldContext);
}

void
DrawHyperCoords (POINT universe)
{
	UNICODE buf[100];

	snprintf (buf, sizeof buf, "%03u.%01u : %03u.%01u",
			universe.x / 10, universe.x % 10,
			universe.y / 10, universe.y % 10);

	DrawSISTitle (buf);
}

// The large blue box in the upper edge of the screen
void
DrawSISMessage (const UNICODE *pStr)
{
	DrawSISMessageEx (pStr, -1, -1, DSME_NONE);
}

// See sis.h for the allowed flags.
BOOLEAN
DrawSISMessageEx (const UNICODE *pStr, SIZE CurPos, SIZE ExPos, COUNT flags)
{
	UNICODE buf[256];
	CONTEXT OldContext;
	TEXT t;
	RECT r;

	OldContext = SetContext (OffScreenContext);
	// prepare the context
	r.corner.x = SIS_ORG_X + 1;
	r.corner.y = SIS_ORG_Y - SIS_MESSAGE_HEIGHT;
	r.extent.width = SIS_MESSAGE_WIDTH;
	r.extent.height = SIS_MESSAGE_HEIGHT - 1;
	SetContextFGFrame (Screen);
	SetContextClipRect (&r);
	
	BatchGraphics ();
	SetContextBackGroundColor (SIS_MESSAGE_BACKGROUND_COLOR);

	if (pStr == 0)
	{
		switch (LOBYTE (GLOBAL (CurrentActivity)))
		{
			default:
			case IN_ENCOUNTER:
				pStr = "";
				break;
			case IN_LAST_BATTLE:
			case IN_INTERPLANETARY:
				GetClusterName (CurStarDescPtr, buf);
				pStr = buf;
				break;
			case IN_HYPERSPACE:
					if (GET_GAME_STATE (ORZ_SPACE_SIDE) > 1)
					{
						pStr = GAME_STRING (NAVIGATION_STRING_BASE + 6);
						// JMS: "ORZ Space = * Below *"
					}
					else 
					{
						pStr = GAME_STRING (NAVIGATION_STRING_BASE
								+ (GET_GAME_STATE (ARILOU_SPACE_SIDE) <= 1
									? 0 : 1));
						// "Hyperspace" or "Quasispace"
					}
					break;
		}

	}

	if (!(flags & DSME_MYCOLOR))
		SetContextForeGroundColor (SIS_MESSAGE_TEXT_COLOR);

	t.baseline.y = SIS_MESSAGE_HEIGHT - RES_CASE(2,6,8); // JMS_GFX
	t.pStr = pStr;
	t.CharCount = (COUNT)~0;
	SetContextFont (TinyFont);

	if (flags & DSME_CLEARFR)
		SetFlashRect (NULL);

	if (CurPos < 0 && ExPos < 0)
	{	// normal state
		ClearDrawable ();
		t.baseline.x = SIS_MESSAGE_WIDTH >> 1;
		t.align = ALIGN_CENTER;
		font_DrawText (&t);
	}
	else
	{	// editing state
		int i;
		RECT text_r;
		// XXX: 128 is currently safe, but it would be better to specify
		//   the size to TextRect()
		BYTE char_deltas[128];
		BYTE *pchar_deltas;

		t.baseline.x = RES_STAT_SCALE(3);
		t.align = ALIGN_LEFT;

		TextRect (&t, &text_r, char_deltas);
		if (text_r.extent.width + t.baseline.x + 2 >= r.extent.width)
		{	// the text does not fit the input box size and so
			// will not fit when displayed later
			// disallow the change
			UnbatchGraphics ();
			SetContextClipRect (NULL);
			SetContext (OldContext);
			return (FALSE);
		}

		ClearDrawable ();

		if (CurPos >= 0 && CurPos <= t.CharCount)
		{	// calc and draw the cursor
			RECT cur_r = text_r;

			for (i = CurPos, pchar_deltas = char_deltas; i > 0; --i)
				cur_r.corner.x += (SIZE)*pchar_deltas++;
			if (CurPos < t.CharCount) /* end of line */
				--cur_r.corner.x;
			
			if (flags & DSME_BLOCKCUR)
			{	// Use block cursor for keyboardless systems
				if (CurPos == t.CharCount)
				{	// cursor at end-line -- use insertion point
					cur_r.extent.width = 1;
				}
				else if (CurPos + 1 == t.CharCount)
				{	// extra pixel for last char margin
					cur_r.extent.width = (SIZE)*pchar_deltas + 2;
				}
				else
				{	// normal mid-line char
					cur_r.extent.width = (SIZE)*pchar_deltas + 1;
				}
			}
			else
			{	// Insertion point cursor
				cur_r.extent.width = 1;
			}
			
			cur_r.corner.y = 0;
			cur_r.extent.height = r.extent.height;
			SetContextForeGroundColor (SIS_MESSAGE_CURSOR_COLOR);
			DrawFilledRectangle (&cur_r);
		}

		SetContextForeGroundColor (SIS_MESSAGE_TEXT_COLOR);

		if (ExPos >= 0 && ExPos < t.CharCount)
		{	// handle extra characters
			t.CharCount = ExPos;
			font_DrawText (&t);

			// print extra chars
			SetContextForeGroundColor (SIS_MESSAGE_EXTRA_TEXT_COLOR);
			for (i = ExPos, pchar_deltas = char_deltas; i > 0; --i)
				t.baseline.x += (SIZE)*pchar_deltas++;
			t.pStr = skipUTF8Chars (t.pStr, ExPos);
			t.CharCount = (COUNT)~0;
			font_DrawText (&t);
		}
		else
		{	// just print the text
			font_DrawText (&t);
		}
	}

	if (flags & DSME_SETFR)
	{
		r.corner.x = 0;
		r.corner.y = 0;
		SetFlashRect (&r);
	}

	UnbatchGraphics ();

	SetContextClipRect (NULL);
	SetContext (OldContext);

	return (TRUE);
}

void
DateToString (char *buf, size_t bufLen,
		BYTE month_index, BYTE day_index, COUNT year_index)
{
	snprintf (buf, bufLen, "%s %02d" STR_MIDDLE_DOT "%04d",
			GAME_STRING (MONTHS_STRING_BASE + month_index - 1),
			day_index, year_index);
}

// The green screen which contains date etc.
void
GetStatusMessageRect (RECT *r)
{
	r->corner.x = RES_STAT_SCALE(2) + RESOLUTION_FACTOR;
	r->corner.y = RES_STAT_SCALE(130) + RES_CASE(0,0,4);
	r->extent.width = STATUS_MESSAGE_WIDTH;
	r->extent.height = STATUS_MESSAGE_HEIGHT;
}

void
DrawStatusMessage (const UNICODE *pStr)
{
	RECT r;
	RECT ctxRect;
	TEXT t;
	UNICODE buf[128];
	CONTEXT OldContext;

	OldContext = SetContext (StatusContext);
	GetContextClipRect (&ctxRect);
	// XXX: Technically, this does not need OffScreenContext. The only reason
	//   it is used is to avoid preserving StatusContext settings.
	SetContext (OffScreenContext);
	SetContextFGFrame (Screen);
	GetStatusMessageRect (&r);
	r.corner.x += ctxRect.corner.x;
	r.corner.y += ctxRect.corner.y;
	SetContextClipRect (&r);

	BatchGraphics ();
	SetContextBackGroundColor (STATUS_MESSAGE_BACKGROUND_COLOR);
	ClearDrawable ();

	if (!pStr)
	{
		if (curMsgMode == SMM_CREDITS)
		{
			snprintf (buf, sizeof buf, "%u %s", MAKE_WORD (
					GET_GAME_STATE (MELNORME_CREDIT0),
					GET_GAME_STATE (MELNORME_CREDIT1)
					), GAME_STRING (STATUS_STRING_BASE + 0)); // "Cr"
		}
		else if (curMsgMode == SMM_RES_UNITS)
		{
			if (GET_GAME_STATE (CHMMR_BOMB_STATE) < 2)
			{
				snprintf (buf, sizeof buf, "%u %s", GLOBAL_SIS (ResUnits),
						GAME_STRING (STATUS_STRING_BASE + 1)); // "RU"
			}
			else
			{
				snprintf (buf, sizeof buf, "%s %s",
						(optWhichMenu == OPT_PC) ?
							GAME_STRING (STATUS_STRING_BASE + 2)
							: STR_INFINITY_SIGN, // "UNLIMITED"
						GAME_STRING (STATUS_STRING_BASE + 1)); // "RU"
			}
		}
		else
		{	// Just a date
			DateToString (buf, sizeof buf,
					GLOBAL (GameClock.month_index),
					GLOBAL (GameClock.day_index),
					GLOBAL (GameClock.year_index));
		}
		pStr = buf;
	}

	t.baseline.x = STATUS_MESSAGE_WIDTH >> 1;
	t.baseline.y = STATUS_MESSAGE_HEIGHT - RES_CASE(1,6,6); // JMS_GFX
	t.align = ALIGN_CENTER;
	t.pStr = pStr;
	t.CharCount = (COUNT)~0;

	SetContextFont (TinyFont);
	SetContextForeGroundColor (STATUS_MESSAGE_TEXT_COLOR);
	font_DrawText (&t);
	UnbatchGraphics ();

	SetContextClipRect (NULL);

	SetContext (OldContext);
}

StatMsgMode
SetStatusMessageMode (StatMsgMode newMode)
{
	StatMsgMode oldMode = curMsgMode;
	curMsgMode = newMode;
	return oldMode;
}

void
DrawCaptainsName (void)
{
	RECT r;
	TEXT t;
	CONTEXT OldContext;
	FONT OldFont;
	Color OldColor;

	OldContext = SetContext (StatusContext);
	OldFont = SetContextFont (TinyFont);
	OldColor = SetContextForeGroundColor (CAPTAIN_NAME_BACKGROUND_COLOR);

	// Background of captain's name.	
	r.corner.x = RES_STAT_SCALE(3);							// JMS_GFX
	r.corner.y = RES_CASE(10,18,30);						// JMS_GFX
	r.extent.width = SHIP_NAME_WIDTH - RES_STAT_SCALE(2);	// JMS_GFX
	r.extent.height = SHIP_NAME_HEIGHT;						// JMS_GFX
	DrawFilledRectangle (&r);

	// The name itself.
	t.baseline.x = (STATUS_WIDTH >> 1) - RES_CASE(1,0,-1);
	t.baseline.y = r.corner.y + RES_CASE(6,10,18); // JMS_GFX
	t.align = ALIGN_CENTER;
	t.pStr = GLOBAL_SIS (CommanderName);
	t.CharCount = (COUNT)~0;
	SetContextForeGroundColor (CAPTAIN_NAME_TEXT_COLOR);
	font_DrawText (&t);

	SetContextForeGroundColor (OldColor);
	SetContextFont (OldFont);
	SetContext (OldContext);
}

void
DrawFlagshipName (BOOLEAN InStatusArea)
{
	RECT r;
	TEXT t;
	FONT OldFont;
	Color OldColor;
	CONTEXT OldContext;
	FRAME OldFontEffect;
	UNICODE buf[250];

	if (InStatusArea)
	{
		OldContext = SetContext (StatusContext);
		OldFont = SetContextFont (StarConFont);

		r.corner.x = RES_CASE(2,4,7);		// JMS_GFX
		r.corner.y = RES_CASE(20,40,62);	// JMS_GFX
		r.extent.width = SHIP_NAME_WIDTH;	// JMS_GFX
		r.extent.height = SHIP_NAME_HEIGHT;

		t.pStr = GLOBAL_SIS (ShipName);
	}
	else
	{
		OldContext = SetContext (SpaceContext);
		OldFont = SetContextFont (MicroFont);

		r.corner.x = RES_CASE(0,24,0);
		r.corner.y = 1;
		r.extent.width = SIS_SCREEN_WIDTH- RES_CASE(0,24,0);
		r.extent.height = SHIP_NAME_HEIGHT + RES_CASE(0,1,0);

		t.pStr = buf;
		snprintf (buf, sizeof buf, "%s %s",
				GAME_STRING (NAMING_STRING_BASE + 1), GLOBAL_SIS (ShipName));
		// XXX: this will not work with UTF-8 strings
		strupr (buf);
	}
	OldFontEffect = SetContextFontEffect (NULL);
	OldColor = SetContextForeGroundColor (FLAGSHIP_NAME_BACKGROUND_COLOR);
	DrawFilledRectangle (&r);

	t.baseline.x = r.corner.x + (r.extent.width >> 1);
	t.baseline.y = r.corner.y + (SHIP_NAME_HEIGHT - 
			(InStatusArea ? RES_CASE(1,3,3) : RES_CASE(0,2,0))); // JMS_GFX
	t.align = ALIGN_CENTER;
	t.CharCount = (COUNT)~0;
	if (optWhichFonts == OPT_PC)
		SetContextFontEffect (SetAbsFrameIndex (FontGradFrame,
				InStatusArea ? 0 : 3));
	else
		SetContextForeGroundColor (THREEDO_FLAGSHIP_NAME_TEXT_COLOR);

	font_DrawText (&t);

	SetContextFontEffect (OldFontEffect);
	SetContextForeGroundColor (OldColor);
	SetContextFont (OldFont);
	SetContext (OldContext);
}

void
DrawFlagshipStats (void)
{
	RECT r;
	TEXT t;
	FONT OldFont;
	Color OldColor;
	FRAME OldFontEffect;
	CONTEXT OldContext;
	UNICODE buf[128];
	SIZE leading;
	BYTE i;
	BYTE energy_regeneration, energy_wait, turn_wait;
	COUNT max_thrust;
	DWORD fuel;

	/* collect stats */
#define ENERGY_REGENERATION 1
#define ENERGY_WAIT 10
#define MAX_THRUST 10
#define TURN_WAIT 17
	energy_regeneration = ENERGY_REGENERATION;
	energy_wait = ENERGY_WAIT;
	max_thrust = MAX_THRUST;
	turn_wait = TURN_WAIT;


	if (GET_GAME_STATE(WHICH_SHIP_PLAYER_HAS) == CHMMR_EXPLORER_SHIP)
	{
	    fuel = EXPLORER_FUEL_CAPACITY;
	}
	else
	{
	    fuel = 10 * FUEL_TANK_SCALE;
	    
	    for (i = 0; i < NUM_MODULE_SLOTS; i++)
		{
			switch (GLOBAL_SIS (ModuleSlots[i])) 
			{
				case FUEL_TANK:
					fuel += FUEL_TANK_CAPACITY;
					break;
				case HIGHEFF_FUELSYS:
					fuel += HEFUEL_TANK_CAPACITY;
					break;
				case DYNAMO_UNIT:
					energy_wait -= 2;
					if (energy_wait < 4)
						energy_wait = 4;
					break;
				case SHIVA_FURNACE:
					energy_regeneration++;
					break;
			}
		}
	}

	for (i = 0; i < NUM_DRIVE_SLOTS; ++i)
		if (GLOBAL_SIS (DriveSlots[i]) == FUSION_THRUSTER)
			max_thrust += 2;

	for (i = 0; i < NUM_JET_SLOTS; ++i)
		if (GLOBAL_SIS (JetSlots[i]) == TURNING_JETS)
			turn_wait -= 2;
	/* END collect stats */

	OldContext = SetContext (SpaceContext);
	OldFont = SetContextFont (StarConFont);
	OldFontEffect = SetContextFontEffect (NULL);
	GetContextFontLeading (&leading);

	/* we need room to play.  full screen width, 4 lines tall */
	r.corner.x = 0;
	r.corner.y = SIS_SCREEN_HEIGHT - (4 * leading);
	r.extent.width = SIS_SCREEN_WIDTH;
	r.extent.height = (4 * leading);

	OldColor = SetContextForeGroundColor (BLACK_COLOR);
	DrawFilledRectangle (&r);

	/*
	   now that we've cleared out our playground, compensate for the
	   fact that the leading is way more than is generally needed.
	*/
	leading -= 3;
	
	if (GET_GAME_STATE(WHICH_SHIP_PLAYER_HAS)!= CHMMR_EXPLORER_SHIP)
	{
		if (RESOLUTION_FACTOR == 0)
			t.baseline.x = SIS_SCREEN_WIDTH / 6; //wild-assed guess, but it worked
		else
			t.baseline.x = SIS_SCREEN_WIDTH / 8; //wild-assed guess number two
	
		t.baseline.y = r.corner.y + leading + 3;
		t.align = ALIGN_RIGHT;
		t.CharCount = (COUNT)~0;

		SetContextFontEffect (SetAbsFrameIndex (FontGradFrame, 4));

		t.pStr = GAME_STRING (FLAGSHIP_STRING_BASE + 0); // "nose:"
		font_DrawText (&t);
		t.baseline.y += leading;
		t.pStr = GAME_STRING (FLAGSHIP_STRING_BASE + 1); // "spread:"
		font_DrawText (&t);
		t.baseline.y += leading;
		t.pStr = GAME_STRING (FLAGSHIP_STRING_BASE + 2); // "side:"
		font_DrawText (&t);
		t.baseline.y += leading;
		t.pStr = GAME_STRING (FLAGSHIP_STRING_BASE + 3); // "tail:"
		font_DrawText (&t);

		t.baseline.x += 5;
		t.baseline.y = r.corner.y + leading + 3;
		t.align = ALIGN_LEFT;
		t.pStr = buf;

		snprintf (buf, sizeof buf, "%-7.7s",
			describeWeapon (GLOBAL_SIS (ModuleSlots[15])));
		font_DrawText (&t);
		t.baseline.y += leading;
		snprintf (buf, sizeof buf,
				  "%-7.7s", describeWeapon (GLOBAL_SIS (ModuleSlots[14])));
		font_DrawText (&t);
		t.baseline.y += leading;
		snprintf (buf, sizeof buf,
				  "%-7.7s", describeWeapon (GLOBAL_SIS (ModuleSlots[13])));
		font_DrawText (&t);
		t.baseline.y += leading;
		snprintf (buf, sizeof buf,
				  "%-7.7s", describeWeapon (GLOBAL_SIS (ModuleSlots[0])));
		font_DrawText (&t);
	}

	t.baseline.x = r.extent.width - 25 - 12 * RESOLUTION_FACTOR; // JMS_GFX
	t.baseline.y = r.corner.y + leading + 3;
	t.align = ALIGN_RIGHT;

	SetContextFontEffect (SetAbsFrameIndex (FontGradFrame, 5));

	t.pStr = GAME_STRING (FLAGSHIP_STRING_BASE + 4); // "maximum velocity:"
	font_DrawText (&t);
	t.baseline.y += leading;
	t.pStr = GAME_STRING (FLAGSHIP_STRING_BASE + 5); // "turning rate:"
	font_DrawText (&t);
	t.baseline.y += leading;
	t.pStr = GAME_STRING (FLAGSHIP_STRING_BASE + 6); // "combat energy:"
	font_DrawText (&t);
	t.baseline.y += leading;
	t.pStr = GAME_STRING (FLAGSHIP_STRING_BASE + 7); // "maximum fuel:"
	font_DrawText (&t);

	t.baseline.x = r.extent.width - (2 << RESOLUTION_FACTOR); // JMS_GFX
	t.baseline.y = r.corner.y + leading + 3;
	t.pStr = buf;

	snprintf (buf, sizeof buf, "%4u", max_thrust * 4);
	font_DrawText (&t);
	t.baseline.y += leading;
	snprintf (buf, sizeof buf, "%4u", 1 + TURN_WAIT - turn_wait);
	font_DrawText (&t);
	t.baseline.y += leading;
	{
		unsigned int energy_per_10_sec =
				(((100 * ONE_SECOND * energy_regeneration) /
				((1 + energy_wait) * BATTLE_FRAME_RATE)) + 5) / 10;
		snprintf (buf, sizeof buf, "%2u.%1u",
				energy_per_10_sec / 10, energy_per_10_sec % 10);
	}
	font_DrawText (&t);
	t.baseline.y += leading;
	snprintf (buf, sizeof buf, "%4u", (fuel / FUEL_TANK_SCALE));
	font_DrawText (&t);

	SetContextFontEffect (OldFontEffect);
	SetContextForeGroundColor (OldColor);
	SetContextFont (OldFont);
	SetContext (OldContext);
}

static const UNICODE *
describeWeapon (BYTE moduleType)
{
	switch (moduleType)
	{
		case GUN_WEAPON:
			return GAME_STRING (FLAGSHIP_STRING_BASE + 8); // "gun"
		case BLASTER_WEAPON:
			return GAME_STRING (FLAGSHIP_STRING_BASE + 9); // "blaster"
		case CANNON_WEAPON:
			return GAME_STRING (FLAGSHIP_STRING_BASE + 10); // "cannon"
		case BOMB_MODULE_0:
		case BOMB_MODULE_1:
		case BOMB_MODULE_2:
		case BOMB_MODULE_3:
		case BOMB_MODULE_4:
		case BOMB_MODULE_5:
			return GAME_STRING (FLAGSHIP_STRING_BASE + 11); // "n/a"
		default:
			return GAME_STRING (FLAGSHIP_STRING_BASE + 12); // "none"
	}
}

void
DrawLanders (void)
{
	BYTE i;
	SIZE width;
	RECT r;
	STAMP s;
	CONTEXT OldContext;

	OldContext = SetContext (StatusContext);

	s.frame = IncFrameIndex (FlagStatFrame);
	GetFrameRect (s.frame, &r);

	i = GLOBAL_SIS (NumLanders);
	r.corner.x = (STATUS_WIDTH >> 1) - r.corner.x;
	s.origin.x = r.corner.x - (((r.extent.width * i) + (2 * (i - 1))) >> 1);
	s.origin.y = RES_STAT_SCALE(29); // JMS_GFX

	width = r.extent.width + 2;
	r.extent.width = (r.extent.width * MAX_LANDERS) + (2 * (MAX_LANDERS - 1)) + 2;
	r.corner.x -= r.extent.width >> 1;
	r.corner.y += s.origin.y;
	SetContextForeGroundColor (BLACK_COLOR);
	DrawFilledRectangle (&r);
	while (i--)
	{
		DrawStamp (&s);
		s.origin.x += width;
	}

	SetContext (OldContext);
}

// Draw the storage bays, below the picture of the flagship.
void
DrawStorageBays (BOOLEAN Refresh)
{
	BYTE i;
	RECT r;
	CONTEXT OldContext;
	COUNT StorageBayCapacity; // JMS
	
	// JMS: Due to popular demand, let's give Explorer a small mineral storage bay!
	if (GET_GAME_STATE(WHICH_SHIP_PLAYER_HAS) == CHMMR_EXPLORER_SHIP)
		StorageBayCapacity = EXPLORER_STORAGE_BAY_CAPACITY;
	else
		StorageBayCapacity = STORAGE_BAY_CAPACITY;

	OldContext = SetContext (StatusContext);

	r.extent.width  = RES_STAT_SCALE(2); // JMS_GFX
	r.extent.height = RES_STAT_SCALE(4); // JMS_GFX
	r.corner.y		= RES_STAT_SCALE(123) + RES_CASE(0,1,5); // JMS_GFX
	
	if (Refresh)
	{
		r.extent.width = NUM_MODULE_SLOTS * (r.extent.width + 1);
		r.corner.x = (STATUS_WIDTH >> 1) - (r.extent.width >> 1);

		SetContextForeGroundColor (BLACK_COLOR);
		DrawFilledRectangle (&r);
		r.extent.width = RES_STAT_SCALE(2); // JMS_GFX
	}
	
	i = (BYTE)CountSISPieces (STORAGE_BAY);
	
	if (i)
	{
		COUNT j;

		r.corner.x = (STATUS_WIDTH >> 1) - ((i * (r.extent.width + 1)) >> 1);
		
		// Draw full bays.
		SetContextForeGroundColor (STORAGE_BAY_FULL_COLOR);
		for (j = GLOBAL_SIS (TotalElementMass); j >= StorageBayCapacity; j -= StorageBayCapacity)
		{
			DrawFilledRectangle (&r);
			r.corner.x += r.extent.width + RES_STAT_SCALE(1); // JMS_GFX;

			--i;
		}

		// This sets how full a partially filled bay is.
		r.extent.height = (RES_STAT_SCALE (4) * j + (StorageBayCapacity - 1)) / StorageBayCapacity;
		
		if (r.extent.height)
		{
			// Filled part of the bay.
			r.corner.y += RES_STAT_SCALE (4) - r.extent.height;
			//r.corner.y += 4 - r.extent.height;
			DrawFilledRectangle (&r);
			
			//
			r.extent.height = RES_STAT_SCALE(4) - r.extent.height;
			// r.extent.height = 4 - r.extent.height;
			if (r.extent.height)
			{
				r.corner.y = RES_STAT_SCALE(123) + RES_CASE(0,1,5);
				SetContextForeGroundColor (STORAGE_BAY_EMPTY_COLOR);
				DrawFilledRectangle (&r);
			}
			
			r.corner.x += r.extent.width + RES_STAT_SCALE(1);

			--i;
		}

		// Draw totally empty bays.
		r.extent.height = RES_STAT_SCALE(4);
		SetContextForeGroundColor (STORAGE_BAY_EMPTY_COLOR);
		while (i--)
		{
			DrawFilledRectangle (&r);
			r.corner.x += r.extent.width + RES_STAT_SCALE(1);
		}
	}

	SetContext (OldContext);
}

void
GetGaugeRect (RECT *pRect, BOOLEAN IsCrewRect)
{
	pRect->extent.width = RES_STAT_SCALE(24) - RES_CASE(0,6,6); // JMS_GFX
	pRect->corner.x = (STATUS_WIDTH >> 1) - (pRect->extent.width >> 1) + RESOLUTION_FACTOR * 2;
	pRect->extent.height = RES_STAT_SCALE(5); // JMS_GFX
	pRect->corner.y = RES_STAT_SCALE((IsCrewRect ? 117 : 38)) + RESOLUTION_FACTOR * 3; // JMS_GFX
}

static void
DrawPC_SIS (void)
{
	TEXT t;
	RECT r;

	GetGaugeRect (&r, FALSE);
	t.baseline.x = STATUS_WIDTH >> 1;
	t.baseline.y = r.corner.y - 1; // JMS_GFX
	t.align = ALIGN_CENTER;
	t.CharCount = (COUNT)~0;
	SetContextFont (TinyFont);
	SetContextForeGroundColor (BLACK_COLOR);

	r.corner.y -= RES_STAT_SCALE(6); // JMS_GFX
	r.corner.x -= RES_STAT_SCALE(1); // JMS_GFX
	r.extent.width += RES_STAT_SCALE(2);
	DrawFilledRectangle (&r);

	SetContextFontEffect (SetAbsFrameIndex (FontGradFrame, 1));
	t.pStr = GAME_STRING (STATUS_STRING_BASE + 3); // "FUEL"
	font_DrawText (&t);

	r.corner.y += RES_STAT_SCALE(79) + (RESOLUTION_FACTOR == 1 ? 2 : 0); // JMS_GFX
	t.baseline.y += RES_STAT_SCALE(79); // JMS_GFX
	DrawFilledRectangle (&r);

	SetContextFontEffect (SetAbsFrameIndex (FontGradFrame, 2));
	t.pStr = GAME_STRING (STATUS_STRING_BASE + 4); // "CREW"
	font_DrawText (&t);
	SetContextFontEffect (NULL);

	// Background of text "CAPTAIN".
	r.corner.x = (2 + 1) << RESOLUTION_FACTOR; // JMS_GFX;
	r.corner.y = RES_STAT_SCALE(3); // JMS_GFX
	r.extent.width = RES_STAT_SCALE(58); // JMS_GFX
	r.extent.height = RES_STAT_SCALE(7); // JMS_GFX
	SetContextForeGroundColor (PC_CAPTAIN_STRING_BACKGROUND_COLOR);
	DrawFilledRectangle (&r);

	// Text "CAPTAIN".
	SetContextForeGroundColor (PC_CAPTAIN_STRING_TEXT_COLOR);
	t.baseline.y = r.corner.y + RES_CASE(6,10,18); // JMS_GFX
	t.pStr = GAME_STRING (STATUS_STRING_BASE + 5); // "CAPTAIN"
	font_DrawText (&t);
}

static void
DrawThrusters (void)
{
	STAMP s;
	COUNT i;

	s.origin.x = RES_STAT_SCALE(1);
	s.origin.y = 0;
	for (i = 0; i < NUM_DRIVE_SLOTS; ++i)
	{
		BYTE which_piece = GLOBAL_SIS (DriveSlots[i]);
		if (which_piece < EMPTY_SLOT)
		{
			s.frame = SetAbsFrameIndex (FlagStatFrame, which_piece + 1 + 0);
			DrawStamp (&s);
			s.frame = IncFrameIndex (s.frame);
			DrawStamp (&s);
		}

		s.origin.y -= RES_STAT_SCALE(3); // JMS_GFX
	}
}

static void
DrawTurningJets (void)
{
	STAMP s;
	COUNT i;

	s.origin.x = RES_STAT_SCALE(1);
	s.origin.y = 0;
	for (i = 0; i < NUM_JET_SLOTS; ++i)
	{
		BYTE which_piece = GLOBAL_SIS (JetSlots[i]);
		if (which_piece < EMPTY_SLOT)
		{
			s.frame = SetAbsFrameIndex (FlagStatFrame, which_piece + 1 + 1);
			DrawStamp (&s);
			s.frame = IncFrameIndex (s.frame);
			DrawStamp (&s);
		}

		s.origin.y -= RES_STAT_SCALE(3); // JMS_GFX
	}
}

static void
DrawModules (void)
{
	STAMP s;
	COUNT i;

	s.origin.x = RES_STAT_SCALE(1); // This properly centers the modules.
	s.origin.y = RES_STAT_SCALE(1);
	for (i = 0; i < NUM_MODULE_SLOTS; ++i)
	{
		BYTE which_piece = GLOBAL_SIS (ModuleSlots[i]);
		if (which_piece < EMPTY_SLOT)
		{
			s.frame = SetAbsFrameIndex (FlagStatFrame, which_piece + 1 + 2);
			DrawStamp (&s);
		}

		s.origin.y -= RES_STAT_SCALE(3); // JMS_GFX
	}
}

// Pre: GraphicsLock is unlocked
static void
DrawSupportShips (void)
{
	HSHIPFRAG hStarShip;
	HSHIPFRAG hNextShip;
	const POINT *pship_pos;
	const POINT ship_pos_1x[MAX_BUILT_SHIPS] =
	{
		SUPPORT_SHIP_PTS_1X
	};
	const POINT ship_pos_2x[MAX_BUILT_SHIPS] =
	{
		SUPPORT_SHIP_PTS_2X
	};
	const POINT ship_pos_4x[MAX_BUILT_SHIPS] =
	{
		SUPPORT_SHIP_PTS_4X
	};
	
	const POINT *ship_pos = RES_CASE(ship_pos_1x,ship_pos_2x,ship_pos_4x);

	for (hStarShip = GetHeadLink (&GLOBAL (built_ship_q)),
			pship_pos = ship_pos;
			hStarShip; hStarShip = hNextShip, ++pship_pos)
	{
		SHIP_FRAGMENT *StarShipPtr;
		STAMP s;

		StarShipPtr = LockShipFrag (&GLOBAL (built_ship_q), hStarShip);
		hNextShip = _GetSuccLink (StarShipPtr);

		s.origin.x = RES_STAT_SCALE(pship_pos->x);
		s.origin.y = RES_STAT_SCALE(pship_pos->y);
		s.frame = SetAbsFrameIndex (StarShipPtr->icons, 2);
		LockMutex (GraphicsLock);
		DrawStamp (&s);
		UnlockMutex (GraphicsLock);

		UnlockShipFrag (&GLOBAL (built_ship_q), hStarShip);
	}
}

static void
DeltaSISGauges_crewDelta (SIZE crew_delta)
{
	if (crew_delta == 0)
		return;

	if (crew_delta != UNDEFINED_DELTA)
	{
		COUNT CrewCapacity;

		if (crew_delta < 0
				&& GLOBAL_SIS (CrewEnlisted) <= (COUNT)-crew_delta)
			GLOBAL_SIS (CrewEnlisted) = 0;
		else
		{
			GLOBAL_SIS (CrewEnlisted) += crew_delta;
			CrewCapacity = GetCrewPodCapacity ();
			if (GLOBAL_SIS (CrewEnlisted) > CrewCapacity)
				GLOBAL_SIS (CrewEnlisted) = CrewCapacity;
		}
	}

	{
		TEXT t;
		UNICODE buf[60];
		RECT r;

		snprintf (buf, sizeof buf, "%u", GLOBAL_SIS (CrewEnlisted));

		GetGaugeRect (&r, TRUE);
		
		t.baseline.x = STATUS_WIDTH >> 1;
		t.baseline.y = r.corner.y + r.extent.height - (RESOLUTION_FACTOR == 1 ? 2 : 0);
		t.align = ALIGN_CENTER;
		t.pStr = buf;
		t.CharCount = (COUNT)~0;

		SetContextForeGroundColor (BLACK_COLOR);
		DrawFilledRectangle (&r);
		SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0x00, 0x0E, 0x00), 0x6C));
		font_DrawText (&t);
	}
}

static void
DeltaSISGauges_fuelDelta (SIZE fuel_delta)
{
	COUNT old_coarse_fuel;
	COUNT new_coarse_fuel;

	if (fuel_delta == 0)
		return;

	if (fuel_delta == UNDEFINED_DELTA)
		old_coarse_fuel = (COUNT)~0;
	else
	{

		old_coarse_fuel = (COUNT)(
				GLOBAL_SIS (FuelOnBoard) / FUEL_TANK_SCALE);
		if (fuel_delta < 0
				&& GLOBAL_SIS (FuelOnBoard) <= (DWORD)-fuel_delta)
		{
			GLOBAL_SIS (FuelOnBoard) = 0;
		}
		else
		{
			DWORD FuelCapacity = GetFuelTankCapacity ();
			GLOBAL_SIS (FuelOnBoard) += fuel_delta;
			if (GLOBAL_SIS (FuelOnBoard) > FuelCapacity)
				GLOBAL_SIS (FuelOnBoard) = FuelCapacity;
		}
	}

	new_coarse_fuel = (COUNT)(GLOBAL_SIS (FuelOnBoard) / FUEL_TANK_SCALE);
	
	if (new_coarse_fuel != old_coarse_fuel)
	{
		TEXT t;
		UNICODE buf[60];
		RECT r;

		snprintf (buf, sizeof buf, "%u", new_coarse_fuel);

		GetGaugeRect (&r, FALSE);
		
		t.baseline.x = STATUS_WIDTH >> 1;
		t.baseline.y = r.corner.y + r.extent.height - (RESOLUTION_FACTOR == 1 ? 2 : 0);
		t.align = ALIGN_CENTER;
		t.pStr = buf;
		t.CharCount = (COUNT)~0;

		SetContextForeGroundColor (BLACK_COLOR);
		DrawFilledRectangle (&r);
		SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0x13, 0x00, 0x00), 0x2C));
		font_DrawText (&t);
	}
}
	
static void
DeltaSISGauges_resunitDelta (SIZE resunit_delta)
{
	if (resunit_delta == 0)
		return;

	if (resunit_delta != UNDEFINED_DELTA)
	{
		if (resunit_delta < 0
				&& GLOBAL_SIS (ResUnits) <= (DWORD)-resunit_delta)
			GLOBAL_SIS (ResUnits) = 0;
		else
			GLOBAL_SIS (ResUnits) += resunit_delta;

		assert (curMsgMode == SMM_RES_UNITS);
	}
	else
	{
		RECT r;

		r.corner.x = RES_STAT_SCALE(2) + RESOLUTION_FACTOR; // JMS_GFX
		r.corner.y = RES_STAT_SCALE(130) + 2 * RESOLUTION_FACTOR; // JMS_GFX
		r.extent.width = STATUS_MESSAGE_WIDTH;
		r.extent.height = STATUS_MESSAGE_HEIGHT;
		SetContextForeGroundColor (
				BUILD_COLOR (MAKE_RGB15 (0x00, 0x08, 0x00), 0x6E));
		DrawFilledRectangle (&r);
	}
		
	DrawStatusMessage (NULL);
}

void
DeltaSISGauges (SIZE crew_delta, SIZE fuel_delta, int resunit_delta)
{
	CONTEXT OldContext;

	if (crew_delta == 0 && fuel_delta == 0 && resunit_delta == 0)
		return;

	OldContext = SetContext (StatusContext);

	BatchGraphics ();
	if (crew_delta == UNDEFINED_DELTA)
	{
		STAMP s;
		s.origin.x = 0;
		s.origin.y = 0;
		s.frame = FlagStatFrame;
		DrawStamp (&s);

		if (optWhichFonts == OPT_PC)
			DrawPC_SIS();

	    if (GET_GAME_STATE(WHICH_SHIP_PLAYER_HAS) == CHMMR_EXPLORER_SHIP)
		{
		}
	    else
		{
			DrawThrusters ();
			DrawTurningJets ();
			DrawModules ();
		}

		UnlockMutex (GraphicsLock);
		DrawSupportShips ();
		LockMutex (GraphicsLock);
	}

	SetContextFont (TinyFont);

	DeltaSISGauges_crewDelta (crew_delta);
	DeltaSISGauges_fuelDelta (fuel_delta);

	if (crew_delta == UNDEFINED_DELTA)
	{
		DrawFlagshipName (TRUE);
		DrawCaptainsName ();
		DrawLanders ();
		DrawStorageBays (FALSE);
	}

	DeltaSISGauges_resunitDelta (resunit_delta);

	UnbatchGraphics ();

	SetContext (OldContext);
}


////////////////////////////////////////////////////////////////////////////
// Crew
////////////////////////////////////////////////////////////////////////////

// Get the total amount of crew aboard the SIS.
COUNT
GetCrewCount (void)
{
	return GLOBAL_SIS (CrewEnlisted);
}

// Get the number of crew which fit in a module of a specified type.
COUNT
GetModuleCrewCapacity (BYTE moduleType)
{
	if (moduleType == CREW_POD)
		return CREW_POD_CAPACITY;

	return 0;
}

// Gets the amount of crew which currently fit in the ship's crew pods.
COUNT
GetCrewPodCapacity (void)
{
	COUNT capacity = 0;
	COUNT slotI;

	if (GET_GAME_STATE(WHICH_SHIP_PLAYER_HAS) == CHMMR_EXPLORER_SHIP)
	{
		capacity = EXPLORER_CREW_CAPACITY;
	}
	else
	{
		for (slotI = 0; slotI < NUM_MODULE_SLOTS; slotI++)
		{
			BYTE moduleType = GLOBAL_SIS (ModuleSlots[slotI]);
			capacity += GetModuleCrewCapacity (moduleType);
		}
	}

	return capacity;
}

// Find the slot number of the crew pod and "seat" number in that crew pod,
// where the Nth crew member would be located.
// If the crew member does not fit, false is returned, and *slotNr and
// *seatNr are unchanged.
static bool
GetCrewPodForCrewMember (COUNT crewNr, COUNT *slotNr, COUNT *seatNr)
{
	COUNT slotI;
	COUNT capacity = 0;

	slotI = NUM_MODULE_SLOTS;
	while (slotI--) {
		BYTE moduleType = GLOBAL_SIS (ModuleSlots[slotI]);
		COUNT moduleCapacity = GetModuleCrewCapacity (moduleType);

		if (crewNr < capacity + moduleCapacity)
		{
			*slotNr = slotI;
			*seatNr = crewNr - capacity;
			return true;
		}
		capacity += moduleCapacity;
	}

	return false;
}

// Get the point where to draw the next crew member,
// set the foreground color to the color for that crew member,
// and return GetCrewPodCapacity ().
// TODO: Split of the parts of this function into separate functions.
// BW: New function for populating the Explorer
COUNT
GetCPodCapacity (POINT *ppt)
{
	if (GET_GAME_STATE(WHICH_SHIP_PLAYER_HAS) == CHMMR_EXPLORER_SHIP)
	{
		int i;
		COUNT line_remainder;
		COORD x,y;

		line_remainder = GetCrewCount() + 1;
		i=0;
  
		while (line_remainder > crew_lines_sizes[i])
		{
			line_remainder -= crew_lines_sizes[i];
			i++;
		}

		switch (RESOLUTION_FACTOR)
		{
			case 2:
				x = crew_lines_x_coords_4x[i];
				break;
			case 1:
				x = crew_lines_x_coords_4x[i];
				break;
			case 0:
			default:
				x = crew_lines_x_coords_1x[i];
				break;
		}			
		y = crew_lines_y_coords[i];
  
		if (ppt)
		{
			static const Color crew_rows[] = PC_EXPLORER_CREW_COLOR_TABLE;
      
			ppt->x = (x + 2 * line_remainder) << RESOLUTION_FACTOR;
			ppt->y = (y << RESOLUTION_FACTOR) + RES_CASE(12,17,34);
      
			if (optWhichFonts == OPT_PC)
				SetContextForeGroundColor (crew_rows[(y-7)/2]);
			else
				SetContextForeGroundColor (THREEDO_CREW_COLOR);
		}
		return GetCrewPodCapacity ();
	}
	else
	{
		COUNT crewCount;
		COUNT slotNr;
		COUNT seatNr;

		COUNT rowNr;
		COUNT colNr;
				
		static const Color crewRows[] = PC_CREW_COLOR_TABLE;

		crewCount = GetCrewCount ();
		if (!GetCrewPodForCrewMember (crewCount, &slotNr, &seatNr))
		{
			// Crew does not fit. *ppt is unchanged.
			return GetCrewPodCapacity ();
		}

		rowNr = seatNr / CREW_PER_ROW;
		colNr = seatNr % CREW_PER_ROW;

		if (optWhichFonts == OPT_PC)
			SetContextForeGroundColor (crewRows[rowNr]);
		else
			SetContextForeGroundColor (THREEDO_CREW_COLOR);
		
		ppt->x = (27 << RESOLUTION_FACTOR) + (slotNr * SHIP_PIECE_OFFSET) - ((colNr * 2) << RESOLUTION_FACTOR); // JMS_GFX
		ppt->y = (34 - (rowNr * 2)) << RESOLUTION_FACTOR; // JMS_GFX

		return GetCrewPodCapacity ();
	}
}


////////////////////////////////////////////////////////////////////////////
// Storage bays
////////////////////////////////////////////////////////////////////////////

// Get the total amount of minerals aboard the SIS.
static COUNT
GetElementMass (void)
{
	return GLOBAL_SIS (TotalElementMass);
}

// Get the number of crew which fit in a module of a specified type.
COUNT
GetModuleStorageCapacity (BYTE moduleType)
{
	COUNT StorageBayCapacity; // JMS
	
	// JMS: Due to popular demand, let's give Explorer a small mineral storage bay!
	if (GET_GAME_STATE(WHICH_SHIP_PLAYER_HAS) == CHMMR_EXPLORER_SHIP)
		StorageBayCapacity = EXPLORER_STORAGE_BAY_CAPACITY;
	else
		StorageBayCapacity = STORAGE_BAY_CAPACITY;
	
	if (moduleType == STORAGE_BAY)
		return StorageBayCapacity;

	return 0;
}

// Gets the amount of minerals which currently fit in the ship's storage.
COUNT
GetStorageBayCapacity (void)
{
	COUNT capacity = 0;
	COUNT slotI;

	for (slotI = 0; slotI < NUM_MODULE_SLOTS; slotI++)
	{
		BYTE moduleType = GLOBAL_SIS (ModuleSlots[slotI]);
		capacity += GetModuleStorageCapacity (moduleType);
	}

	return capacity;
}

// Find the slot number of the storage bay and "storage cell" number in that
// storage bay, where the N-1th mineral unit would be located.
// If the mineral unit does not fit, false is returned, and *slotNr and
// *cellNr are unchanged.
static bool
GetStorageCellForMineralUnit (COUNT unitNr, COUNT *slotNr, COUNT *cellNr)
{
	COUNT slotI;
	COUNT capacity = 0;

	slotI = NUM_MODULE_SLOTS;
	while (slotI--) {
		BYTE moduleType = GLOBAL_SIS (ModuleSlots[slotI]);
		COUNT moduleCapacity = GetModuleStorageCapacity (moduleType);

		if (unitNr <= capacity + moduleCapacity)
		{
			*slotNr = slotI;
			*cellNr = unitNr - capacity;
			return true;
		}
		capacity += moduleCapacity;
	}

	return false;
}

// Get the point where to draw the next mineral unit,
// set the foreground color to the color for that mineral unit,
// and return GetStorageBayCapacity ().
// TODO: Split of the parts of this function into separate functions.
COUNT
GetSBayCapacity (POINT *ppt)
{
	COUNT massCount;
	COUNT slotNr;
	COUNT cellNr;

	COUNT rowNr;
	COUNT colNr;
				
	static const Color colorBars[] = STORAGE_BAY_COLOR_TABLE;

	massCount = GetElementMass ();
	if (!GetStorageCellForMineralUnit (massCount, &slotNr, &cellNr))
	{
		// Crew does not fit. *ppt is unchanged.
		return GetStorageBayCapacity ();
	}

	rowNr = cellNr / SBAY_MASS_PER_ROW;
	colNr = cellNr % SBAY_MASS_PER_ROW;

	if (rowNr == 0)
		SetContextForeGroundColor (BLACK_COLOR);
	else
	{
		rowNr--;
		SetContextForeGroundColor (colorBars[rowNr]);
	}
		
	ppt->x = (19 << RESOLUTION_FACTOR) + (slotNr * SHIP_PIECE_OFFSET);
	ppt->y = (34 - (rowNr * 2)) << RESOLUTION_FACTOR;
	
	// Graphics positioning for tentative Explorer storage bay
	if (GET_GAME_STATE(WHICH_SHIP_PLAYER_HAS) == CHMMR_EXPLORER_SHIP)
	{
		ppt->y = (27 - rowNr) << RESOLUTION_FACTOR;
		if (rowNr == 9)
			ppt->x = (40 << RESOLUTION_FACTOR);
	}

	return GetStorageBayCapacity ();
}


////////////////////////////////////////////////////////////////////////////
// Fuel tanks
////////////////////////////////////////////////////////////////////////////

// Get the total amount of fuel aboard the SIS.
static DWORD
GetFuelTotal (void)
{
	return GLOBAL_SIS (FuelOnBoard);
}

// Get the amount of fuel which fits in a module of a specified type.
DWORD
GetModuleFuelCapacity (BYTE moduleType)
{
	if (moduleType == FUEL_TANK)
		return FUEL_TANK_CAPACITY;

	if (moduleType == HIGHEFF_FUELSYS)
		return HEFUEL_TANK_CAPACITY;

	return 0;
}

// Gets the amount of fuel which currently fits in the ship's fuel tanks.
DWORD
GetFuelTankCapacity (void)
{
	if (GET_GAME_STATE(WHICH_SHIP_PLAYER_HAS) == CHMMR_EXPLORER_SHIP)
	{
		return EXPLORER_FUEL_CAPACITY;
	}
	else
	{
		DWORD capacity = FUEL_RESERVE;
		COUNT slotI;

		for (slotI = 0; slotI < NUM_MODULE_SLOTS; slotI++)
		{
			BYTE moduleType = GLOBAL_SIS (ModuleSlots[slotI]);
			capacity += GetModuleFuelCapacity (moduleType);
		}

		return capacity;
	}
}

// Find the slot number of the fuel cell and "compartment" number in that
// crew pod, where the Nth unit of fuel would be located.
// If the unit does not fit, false is returned, and *slotNr and
// *compartmentNr are unchanged.
// Pre: unitNr >= FUEL_RESERER
static bool
GetFuelTankForFuelUnit (DWORD unitNr, COUNT *slotNr, DWORD *compartmentNr)
{
	COUNT slotI;
	DWORD capacity = FUEL_RESERVE;

	assert (unitNr >= FUEL_RESERVE);

	slotI = NUM_MODULE_SLOTS;
	while (slotI--) {
		BYTE moduleType = GLOBAL_SIS (ModuleSlots[slotI]);
	
		capacity += GetModuleFuelCapacity (moduleType);
		if (unitNr < capacity)
		{
			*slotNr = slotI;
			*compartmentNr = capacity - unitNr;
			return true;
		}
	}

	return false;
}

// Get the point where to draw the next fuel unit,
// set the foreground color to the color for that unit,
// and return GetFuelTankCapacity ().
// TODO: Split of the parts of this function into separate functions.
// BW: new function for fueling the Explorer
DWORD
GetFTankCapacity (POINT *ppt)
{
	DWORD capacity;

	capacity = GetFuelTankCapacity();

	if (GET_GAME_STATE(WHICH_SHIP_PLAYER_HAS) == CHMMR_EXPLORER_SHIP)
	{
		if (ppt)
		{
			COUNT which_row;
			static const Color fuel_colors[] = FUEL_COLOR_TABLE;
	  
			which_row = (COUNT)(GetFuelTotal() * EXPLORER_MAX_FUEL_BARS
					    / (EXPLORER_FUEL_CAPACITY));
	
			ppt->x = (31 << RESOLUTION_FACTOR) + which_row; // JMS_GFX
			ppt->y = RES_CASE(22, 48, 97); // JMS_GFX
		
			which_row = (COUNT)(GetFuelTotal() * 10 / (EXPLORER_FUEL_CAPACITY));
				
			SetContextForeGroundColor (fuel_colors[which_row]);
			SetContextBackGroundColor (fuel_colors[which_row + 1]);
		}
	}
	else
	{
		DWORD fuelAmount;
		COUNT slotNr;
		DWORD compartmentNr;
		BYTE moduleType;
		DWORD volume;

		COUNT rowNr;
				
		static const Color fuelColors[] = FUEL_COLOR_TABLE;

		fuelAmount = GetFuelTotal ();
		if (fuelAmount < FUEL_RESERVE)
		{
			// Fuel is in the SIS reserve, not in a fuel tank.
			// *ppt is unchanged
			return capacity;
		}
		if (!GetFuelTankForFuelUnit (fuelAmount, &slotNr, &compartmentNr))
		{
			// Fuel does not fit. *ppt is unchanged.
			return capacity;
		}

		moduleType = GLOBAL_SIS (ModuleSlots[slotNr]);
		volume = GetModuleFuelCapacity (moduleType);

		rowNr = ((volume - compartmentNr) * MAX_FUEL_BARS / HEFUEL_TANK_CAPACITY);
		
		ppt->x = (21 << RESOLUTION_FACTOR) + (slotNr * SHIP_PIECE_OFFSET);
		if (volume == FUEL_TANK_CAPACITY)
			ppt->y = (27 << RESOLUTION_FACTOR) - rowNr;
		else
			ppt->y = (30 << RESOLUTION_FACTOR) - rowNr;

		assert (rowNr + 1 < (COUNT) (sizeof fuelColors / sizeof fuelColors[0]));
		SetContextForeGroundColor (fuelColors[rowNr]);
		SetContextBackGroundColor (fuelColors[rowNr + 1]);
	}

	return capacity;
}


////////////////////////////////////////////////////////////////////////////

COUNT
CountSISPieces (BYTE piece_type)
{
	COUNT i, num_pieces;

	num_pieces = 0;
	if (piece_type == FUSION_THRUSTER)
	{
		for (i = 0; i < NUM_DRIVE_SLOTS; ++i)
		{
			if (GLOBAL_SIS (DriveSlots[i]) == piece_type)
				++num_pieces;
		}
	}
	else if (piece_type == TURNING_JETS)
	{
		for (i = 0; i < NUM_JET_SLOTS; ++i)
		{
			if (GLOBAL_SIS (JetSlots[i]) == piece_type)
				++num_pieces;
		}
	}
	else
	{
		for (i = 0; i < NUM_MODULE_SLOTS; ++i)
		{
			if (GLOBAL_SIS (ModuleSlots[i]) == piece_type)
				++num_pieces;
		}
	}

	return num_pieces;
}

void
DrawAutoPilotMessage (BOOLEAN Reset)
{
	static BOOLEAN LastPilot = FALSE;
	static TimeCount NextTime = 0;
	static DWORD cycle_index = 0;
	BOOLEAN OnAutoPilot;
	
	static const Color cycle_tab[] = AUTOPILOT_COLOR_CYCLE_TABLE;
	const size_t cycleCount = sizeof cycle_tab / sizeof cycle_tab[0];
#define BLINK_RATE (ONE_SECOND * 3 / 40) // 9 @ 120 ticks/second

	if (Reset)
	{	// Just a reset, not drawing
		LastPilot = FALSE;
		return;
	}

	OnAutoPilot = (GLOBAL (autopilot.x) != ~0 && GLOBAL (autopilot.y) != ~0)
			|| GLOBAL_SIS (FuelOnBoard) == 0;

	if (OnAutoPilot || LastPilot)
	{
		if (!OnAutoPilot)
		{	// AutoPilot aborted -- clear the AUTO-PILOT message
			DrawSISMessage (NULL);
			cycle_index = 0;
		}
		else if (GetTimeCounter () >= NextTime)
		{
			if (!(GLOBAL (CurrentActivity) & CHECK_ABORT)
					&& GLOBAL_SIS (CrewEnlisted) != (COUNT)~0)
			{
				CONTEXT OldContext;

				OldContext = SetContext (OffScreenContext);
				SetContextForeGroundColor (cycle_tab[cycle_index]);
				if (GLOBAL_SIS (FuelOnBoard) == 0)
				{
					DrawSISMessageEx (GAME_STRING (NAVIGATION_STRING_BASE + 2),
							-1, -1, DSME_MYCOLOR);   // "OUT OF FUEL"
				}
				else
				{
					DrawSISMessageEx (GAME_STRING (NAVIGATION_STRING_BASE + 3),
							-1, -1, DSME_MYCOLOR);   // "AUTO-PILOT"
				}
				SetContext (OldContext);
			}

			cycle_index = (cycle_index + 1) % cycleCount;
			NextTime = GetTimeCounter () + BLINK_RATE;
		}

		LastPilot = OnAutoPilot;
	}
}


Task flash_task = 0;
RECT flash_rect;
static FRAME flash_screen_frame = 0;
		// The original contents of the flash rectangle.
static int flash_changed;
Mutex flash_mutex = 0;

static int
flash_rect_func (void *data)
{
#define NORMAL_STRENGTH 4
#define NORMAL_F_STRENGTH 0
#define CACHE_SIZE 10
	DWORD TimeIn;
	const DWORD WaitTime = ONE_SECOND / 16;
	SIZE strength;
	RECT cached_rect = {{0, 0}, {0, 0}};
	FRAME cached_screen_frame = 0;
	Task task = (Task)data;
	bool cached[CACHE_SIZE];
	STAMP cached_stamp[CACHE_SIZE];
	int i;

	// Init cache
	for (i = 0; i < CACHE_SIZE; i++)
	{
		cached[i] = false;
		cached_stamp[i].frame = 0;
	}

	strength = NORMAL_STRENGTH;
	TimeIn = GetTimeCounter ();
	while (!Task_ReadState(task, TASK_EXIT))
	{
		CONTEXT OldContext;

		LockMutex (flash_mutex);
		if (flash_changed)
		{
			cached_rect = flash_rect;
			if (cached_screen_frame)
				DestroyDrawable (ReleaseDrawable (cached_screen_frame));
			flash_changed = 0;
			//  Wait for the  flash_screen_frame to get initialized
			FlushGraphics ();
			cached_screen_frame = CaptureDrawable (
					CloneFrame (flash_screen_frame));
			UnlockMutex (flash_mutex);

			// Clear the cache.
			for (i = 0; i < CACHE_SIZE; i++)
			{
				cached[i] = false;
				if (cached_stamp[i].frame)
					DestroyDrawable (ReleaseDrawable (cached_stamp[i].frame));
				cached_stamp[i].frame = 0;
			}
		}
		else
			UnlockMutex (flash_mutex);
		
		if (cached_rect.extent.width)
		{
			STAMP *pStamp;

#define MIN_STRENGTH 4
#define MAX_STRENGTH 6
			strength += 2;
			if (strength > MAX_STRENGTH)
				strength = MIN_STRENGTH;
			if (cached[strength - MIN_STRENGTH])
				pStamp = &cached_stamp[strength - MIN_STRENGTH];
			else
			{
				pStamp = &cached_stamp[strength - MIN_STRENGTH];
				cached[strength - MIN_STRENGTH] = true;
				pStamp->frame = CaptureDrawable (
						CloneFrame (cached_screen_frame));
				pStamp->origin.x = 0;
				pStamp->origin.y = 0;

				if (strength != 4)
				{	// brighten the frame with an additive
					DrawMode oldMode;
					STAMP s;
					int factor;

					s.origin.x = 0;
					s.origin.y = 0;
					s.frame = cached_screen_frame;

					factor = (strength - MIN_STRENGTH) * DRAW_FACTOR_1 / 4;

					LockMutex (GraphicsLock);
					OldContext = SetContext (OffScreenContext);
					SetContextFGFrame (pStamp->frame);
					SetContextClipRect (NULL);
					oldMode	= SetContextDrawMode (MAKE_DRAW_MODE (
							DRAW_ADDITIVE, factor));
					DrawStamp (&s);
					SetContextDrawMode (oldMode);
					SetContext (OldContext);
					UnlockMutex (GraphicsLock);
				}
			}

			LockMutex (GraphicsLock);
			OldContext = SetContext (ScreenContext);
			SetContextClipRect (&cached_rect);
			// flash changed_can't be modified while GraphicSem is held
			if (!flash_changed)
				DrawStamp (pStamp);
			// XXX: Shouldn't we save and restore the original cliprect?
			SetContextClipRect (NULL);
			SetContext (OldContext);
			UnlockMutex (GraphicsLock);
		}
		FlushGraphics ();
		SleepThreadUntil (TimeIn + WaitTime);
		TimeIn = GetTimeCounter ();
	}

	// Clear cache
	{
		if (cached_screen_frame)
			DestroyDrawable (ReleaseDrawable (cached_screen_frame));

		for (i = 0; i < CACHE_SIZE; i++)
		{
			if(cached_stamp[i].frame)
				DestroyDrawable (ReleaseDrawable (cached_stamp[i].frame));
		}
	}
	LockMutex (flash_mutex);
	flash_task = 0;
	UnlockMutex (flash_mutex);

	FinishTask (task);
	return 0;
}

void
SetFlashRect (RECT *pRect)
{
	RECT clip_r = {{0, 0}, {0, 0}};
	RECT temp_r, flash_rect1, old_r;
	CONTEXT OldContext;
	int create_flash = 0;

	if (!flash_mutex)
		flash_mutex = CreateMutex ("FlashRect Lock",
				SYNC_CLASS_TOPLEVEL | SYNC_CLASS_VIDEO);

	old_r = flash_rect;
	flash_rect1 = flash_rect;
		
	if (pRect != SFR_MENU_3DO && pRect != SFR_MENU_ANY)
	{
		GetContextClipRect (&clip_r);
		OldContext = SetContext (ScreenContext);
	}
	else
	{
		//Don't flash when using the PC menu
 		if (optWhichMenu == OPT_PC && pRect != SFR_MENU_ANY)
 		{
 			OldContext = SetContext (ScreenContext);
 			pRect = 0;
 		}
 		else
 		{
 			OldContext = SetContext (StatusContext);
 			GetContextClipRect (&clip_r);
 			pRect = &temp_r;
 			temp_r.corner.x = RADAR_X - clip_r.corner.x;
 			temp_r.corner.y = RADAR_Y - clip_r.corner.y;
 			temp_r.extent.width = RADAR_WIDTH;
 			temp_r.extent.height = RADAR_HEIGHT;
 			SetContext (ScreenContext);
		}
	}

	if (pRect == 0 || pRect->extent.width == 0)
	{
		// End the flashing.
		flash_rect1.extent.width = 0;
		if (flash_task)
		{
			UnlockMutex (GraphicsLock);
			ConcludeTask (flash_task);
			LockMutex (GraphicsLock);
		}
	}
	else
	{
		flash_rect1 = *pRect;
		flash_rect1.corner.x += clip_r.corner.x;
		flash_rect1.corner.y += clip_r.corner.y;
		create_flash = 1;
	}
	
	LockMutex (flash_mutex);
	flash_rect = flash_rect1;

	if (old_r.extent.width && !rectsEqual (old_r, flash_rect))
	{
		// We had a flash rectangle, and now a different one is set.
		if (flash_screen_frame)
		{
			// The screen contents may have changed; we grab a new copy.
			STAMP old_s;
			old_s.origin.x = old_r.corner.x;
			old_s.origin.y = old_r.corner.y;
			old_s.frame = flash_screen_frame;
			DrawStamp (&old_s);
			DestroyDrawable (ReleaseDrawable (flash_screen_frame));
			flash_screen_frame = 0;
		}
		else
			log_add (log_Debug, "Couldn't locate flash_screen_rect");
	}
	
	if (flash_rect.extent.width)
	{
		// A new flash rectangle is set.
		// Copy the original contents of the rectangle from the screen.
		if (flash_screen_frame)
			DestroyDrawable (ReleaseDrawable (flash_screen_frame));
		flash_screen_frame =
				CaptureDrawable (LoadDisplayPixmap (&flash_rect, (FRAME)0));
	}
	flash_changed = 1;
	UnlockMutex (flash_mutex);
	// we create the thread after the LoadDisplayPixmap()
	// so there is no race between the FlushGraphics in flash_task
	// and the Enqueue in LoadDisplayPixmap()
	if (create_flash && flash_task == 0)
	{
		flash_task = AssignTask (flash_rect_func, 2048,
				"flash rectangle");
	}

	SetContext (OldContext);
}

